import unittest
from unittest.mock import MagicMock

from mermaid.configuration import Config, Themes
from mermaid.userjourney import Actor, Section, Task, UserJourney


class TestTask(unittest.TestCase):
    def test_str_task_with_one_actor(self):
        actors = Actor("Me")
        task = Task("Make tea", 5, actors)
        expect_string = "\t\tMake tea: 5 : Me"
        self.assertEqual(expect_string, str(task))

    def test_str_task_with_actors(self):
        actors = [Actor("Me"), Actor("Cat")]
        task = Task("Make tea", 5, actors)
        expect_string = "\t\tMake tea: 5 : Me, Cat"
        self.assertEqual(expect_string, str(task))


class TestSection(unittest.TestCase):
    def test_string_section(self):
        task_mock_1 = MagicMock(spec=Task)
        task_mock_2 = MagicMock(spec=Task)
        task_mock_1.__str__.return_value = "\t\tMake tea1: 5 : Me, Cat"
        task_mock_2.__str__.return_value = "\t\tMake tea2: 5 : Me, Cat"
        section = Section("My working day", [task_mock_1, task_mock_2])
        expect_string = """\tsection My working day
\t\tMake tea1: 5 : Me, Cat
\t\tMake tea2: 5 : Me, Cat
"""
        self.assertEqual(expect_string, str(section))


class TestUserJourney(unittest.TestCase):
    def test_script_with_task(self):
        task_mock_1 = MagicMock(spec=Task)
        task_mock_2 = MagicMock(spec=Task)
        task_mock_1.__str__.return_value = "\tMake tea1: 5 : Me, Cat"
        task_mock_2.__str__.return_value = "\tMake tea2: 5 : Me, Cat"
        userjourney = UserJourney("simple user journey", [task_mock_1, task_mock_2])
        expect_string = """---
title: simple user journey
---
journey
\ttitle simple user journey
\tMake tea1: 5 : Me, Cat
\tMake tea2: 5 : Me, Cat
"""
        self.assertEqual(expect_string, userjourney.script)

    def test_script_with_section(self):
        section_mock_1 = MagicMock(spec=Section)
        section_mock_2 = MagicMock(spec=Section)
        section_mock_1.__str__.return_value = """\tsection section-1
\t\tMake tea1: 5 : Me, Cat
\t\tMake tea2: 5 : Me, Cat
"""
        section_mock_2.__str__.return_value = """\tsection section-2
\t\tMake tea1: 5 : Me, Cat
\t\tMake tea2: 5 : Me, Cat
"""
        userjourney = UserJourney(
            "simple user journey", [section_mock_1, section_mock_2]
        )
        expect_string = """---
title: simple user journey
---
journey
\ttitle simple user journey
\tsection section-1
\t\tMake tea1: 5 : Me, Cat
\t\tMake tea2: 5 : Me, Cat

\tsection section-2
\t\tMake tea1: 5 : Me, Cat
\t\tMake tea2: 5 : Me, Cat

"""
        self.assertEqual(expect_string, userjourney.script)

    def test_script_wth_config(self):
        config_mock = Config(theme=Themes.FOREST)
        userjourney = UserJourney("simple user journey", [], config_mock)
        expect_string = """---
title: simple user journey
---
%%{
\tinit: {
\t\t"theme": "forest",
\t\t"themeVariables": {
\t\t}
\t}
}%%

journey
\ttitle simple user journey
"""
        self.assertEqual(expect_string, userjourney.script)



import os
import unittest
from pathlib import Path

from mermaid import Mermaid, Position
from mermaid.graph import Graph


class TestMermaid(unittest.TestCase):
    def setUp(self) -> None:
        self.script: str = """graph TD;
    A-->B;
    A-->C;
    B-->D;
    C-->D;"""
        self.name: str = "simple-graph"
        self.graph: Graph = Graph(self.name, self.script)
        self.mermaid_object = Mermaid(self.graph)

    def test_make_request_to_mermaid_api_for_svg(self):
        self.assertTrue(self.mermaid_object.svg_response.status_code == 200)

    def test_make_request_to_mermaid_api_for_png(self):
        self.assertTrue(self.mermaid_object.img_response.status_code == 200)

    def test_using_mermaid_with_str_script(self):
        mermaid_with_str_script = Mermaid(self.script)
        self.assertTrue(mermaid_with_str_script.svg_response.status_code == 200)
        self.assertTrue(mermaid_with_str_script.img_response.status_code == 200)

    def test_query_params_to_mermaid_api_for_svg(self):
        mermaid_with_params = Mermaid(self.graph, height=1024, scale=2)
        self.assertTrue(mermaid_with_params.img_response.status_code == 200)

    def test_should_raise_error_on_invalid_scale(self):
        with self.assertRaises(AssertionError):
            Mermaid(self.graph, scale=2)

        with self.assertRaises(AssertionError):
            Mermaid(self.graph, height=100, scale=4)

        with self.assertRaises(AssertionError):
            Mermaid(self.graph, width=100, scale=0.5)

    def test_to_svg_on_mermaid(self):
        output_path: str = f"./{self.name}.svg"
        self.mermaid_object.to_svg(output_path)

        self.assertTrue(os.path.exists(output_path))

    def test_to_svg_on_mermaid_with_path(self):
        output_path: Path = Path(f"./{self.name}.svg")
        self.mermaid_object.to_svg(output_path)

        self.assertTrue(Path.exists(output_path))

    def test_to_png_on_mermaid(self):
        output_path: str = f"./{self.name}.png"
        self.mermaid_object.to_png(output_path)

        self.assertTrue(os.path.exists(output_path))

    def test_to_png_on_mermaid_with_path(self):
        output_path: Path = Path(f"./{self.name}.png")
        self.mermaid_object.to_png(output_path)

        self.assertTrue(Path.exists(output_path))

    def test_repr_html_on_mermaid_with_default_position(self):
        self.assertEqual(
            self.mermaid_object._repr_html_(), self.mermaid_object.svg_response.text
        )

    def test_repr_html_on_mermaid_with_str_position(self):
        position = "center"
        self.mermaid_object.set_position(position)
        self.assertTrue(
            self.mermaid_object._repr_html_().startswith(
                f'<div style="text-align:{position}">'
            )
        )
        self.assertTrue(self.mermaid_object._repr_html_().endswith("</div>"))

    def test_repr_html_on_mermaid_with_enum_position(self):
        position = Position.RIGHT
        self.mermaid_object.set_position(position)
        self.assertTrue(
            self.mermaid_object._repr_html_().startswith(
                f'<div style="text-align:{position.value}">'
            )
        )
        self.assertTrue(self.mermaid_object._repr_html_().endswith("</div>"))

    def tearDown(self) -> None:
        output_svg: str = f"./{self.name}.svg"
        output_png: str = f"./{self.name}.png"
        output_svg_path: Path = Path(f"./{self.name}.svg")
        output_png_path: Path = Path(f"./{self.name}.png")

        if os.path.exists(output_svg):
            os.remove(output_svg)
        if os.path.exists(output_png):
            os.remove(output_png)
        if os.path.exists(output_svg_path):
            os.remove(output_svg_path)
        if os.path.exists(output_png_path):
            os.remove(output_png_path)

        return super().tearDown()



import unittest

from mermaid import Config
from mermaid.icon import Icon
from mermaid.mindmap import Level, LevelShape, Mindmap


class TestLevelShape(unittest.TestCase):
    def test_level_shape(self):
        self.assertEqual(LevelShape.SQUARE.end, "]")
        self.assertEqual(LevelShape.SQUARE.start, "[")
        self.assertEqual(LevelShape.ROUNDED_SQUARE.end, ")")
        self.assertEqual(LevelShape.ROUNDED_SQUARE.start, "(")
        self.assertEqual(LevelShape.CIRCLE.end, "))")
        self.assertEqual(LevelShape.CIRCLE.start, "((")
        self.assertEqual(LevelShape.BANG.end, "((")
        self.assertEqual(LevelShape.BANG.start, "))")
        self.assertEqual(LevelShape.CLOUD.end, "(")
        self.assertEqual(LevelShape.CLOUD.start, ")")
        self.assertEqual(LevelShape.HEXAGON.end, "}}")
        self.assertEqual(LevelShape.HEXAGON.start, "{{")
        self.assertEqual(LevelShape.DEFAULT.end, "")
        self.assertEqual(LevelShape.DEFAULT.start, "")


class TestLevel(unittest.TestCase):
    def test_level(self):
        level = Level("name")
        self.assertEqual(level.id_, "name")
        self.assertEqual(level.name, "name")
        self.assertEqual(level.children, [])
        self.assertEqual(level.shape, LevelShape.DEFAULT)

    def test_add_child(self):
        level = Level("name")
        level.add_child(Level("child"))
        self.assertEqual(level.children[0].name, "child")

    def test_str(self):
        level = Level("name")
        self.assertEqual(str(level), "\t\tname\n")

    def test_list_str(self):
        level = Level("name")
        level.add_child(Level("child"))
        self.assertEqual(level.list_str(), ["name", ["child"]])

    def test_list_str_with_level_deapth_2(self):
        level = Level("name")
        level.add_child(Level("child"))
        level.children[0].add_child(Level("grandchild", shape=LevelShape.CIRCLE))
        level.children[0].add_child(Level("grandchild2", shape=LevelShape.SQUARE))
        self.assertEqual(
            level.list_str(), ["name", ["child", ["((grandchild))", "[grandchild2]"]]]
        )

    def test_list_str_with_level_deapth_3(self):
        level = Level("name")
        level.add_child(Level("child"))
        level.children[0].add_child(Level("grandchild"))
        level.children[0].children[0].add_child(Level("greatgrandchild"))
        level.children[0].add_child(Level("grandchild2"))
        self.assertEqual(
            level.list_str(),
            ["name", ["child", ["grandchild", ["greatgrandchild"], "grandchild2"]]],
        )

    def test_str_with_deapth_2(self):
        level = Level("name")
        level.add_child(Level("child"))
        level.children[0].add_child(Level("grandchild", shape=LevelShape.CIRCLE))
        level.children[0].add_child(Level("grandchild2", shape=LevelShape.SQUARE))
        expected = (
            "\t\tname\n\t\t\tchild\n\t\t\t\t((grandchild))\n\t\t\t\t[grandchild2]\n"
        )
        self.assertEqual(str(level), expected)

    def test_str_with_deapth_3(self):
        level = Level("name")
        level.add_child(Level("child"))
        level.children[0].add_child(Level("grandchild"))
        level.children[0].children[0].add_child(Level("greatgrandchild"))
        level.children[0].add_child(Level("grandchild2"))
        expected = "\t\tname\n\t\t\tchild\n\t\t\t\tgrandchild\n\t\t\t\t\tgreatgrandchild\n\t\t\t\tgrandchild2\n"
        self.assertEqual(str(level), expected)

    def test_str_with_icon(self):
        level = Level("name", icon=Icon("icon", "fa"))
        expected = "\t\tname\n\t\t::icon(fa icon)\n"
        self.assertEqual(str(level), expected)

    def test_str_with_icon_and_deapth_2(self):
        level = Level("name")
        level.add_child(Level("child"))
        level.children[0].add_child(
            Level("grandchild", shape=LevelShape.CIRCLE, icon=Icon("icon", "fa"))
        )
        level.children[0].add_child(Level("grandchild2", shape=LevelShape.SQUARE))
        expected = """\t\tname
\t\t\tchild
\t\t\t\t((grandchild))
\t\t\t\t::icon(fa icon)
\t\t\t\t[grandchild2]
"""
        self.assertEqual(str(level), expected)


class TestMindmap(unittest.TestCase):
    def setUp(self):
        self.level_depth_1 = Level("name")
        self.level_depth_2 = Level("name 2", [Level("child")])
        self.level_depth_3 = Level("name 3", [Level("child", [Level("grandchild")])])

    def test_mindmap(self):
        mindmap = Mindmap("title")
        self.assertEqual(mindmap.title, "title")
        self.assertEqual(mindmap.levels, [])
        self.assertEqual(mindmap.shape, LevelShape.DEFAULT)

    def test_mindmap_with_levels(self):
        mindmap = Mindmap("title", levels=[self.level_depth_1])

        expecte_script = """---
title: title
---
mindmap
\ttitle
\t\tname
"""
        self.assertEqual(mindmap.script, expecte_script)

    def test_mindmap_with_levels_and_shape(self):
        mindmap = Mindmap("title", levels=[self.level_depth_1], shape=LevelShape.CIRCLE)

        expecte_script = """---
title: title
---
mindmap
\t((title))
\t\tname
"""
        self.assertEqual(mindmap.script, expecte_script)

    def test_mindmap_with_levels_depth_2(self):
        mindmap = Mindmap("title", levels=[self.level_depth_2])

        expecte_script = """---
title: title
---
mindmap
\ttitle
\t\tname 2
\t\t\tchild
"""
        self.assertEqual(mindmap.script, expecte_script)

    def test_mindmap_with_levels_depth_3(self):
        mindmap = Mindmap("title", levels=[self.level_depth_3])

        expecte_script = """---
title: title
---
mindmap
\ttitle
\t\tname 3
\t\t\tchild
\t\t\t\tgrandchild
"""
        self.assertEqual(mindmap.script, expecte_script)

    def test_mindmap_with_config(self):
        config = Config(primary_color="red")
        mindmap = Mindmap("title", config=config)

        expecte_script = """---
title: title
---
%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t\t"primaryColor": "red"
\t\t}
\t}
}%%

mindmap
\ttitle
"""
        self.assertEqual(mindmap.script, expecte_script)



import unittest
from unittest import mock

from mermaid.configuration import Config
from mermaid.sequence import SequenceDiagram
from mermaid.sequence.element import Actor, Box, Note, NotePosition, Participant, Rect
from mermaid.sequence.link import ArrowTypes, Link
from mermaid.sequence.logic import Alt, Break, Critical, Loop, Optional, Parallel


class TestActor(unittest.TestCase):
    def test_str(self):
        actor = Actor("John")
        expect_str = "\tactor John\n"
        self.assertEqual(str(actor), expect_str)
        self.assertEqual(actor.id_, "John")


class TestParticipant(unittest.TestCase):
    def test_str(self):
        participant = Participant("Alice")
        expect_str = "\tparticipant alice as Alice\n"
        self.assertEqual(str(participant), expect_str)


class TestBox(unittest.TestCase):
    def test_str(self):
        # use mocks to create an actor and a participant element
        actor = mock.MagicMock(spec=Actor)
        actor.__str__ = mock.Mock(return_value="\tActor John\n")
        participant = mock.MagicMock(spec=Participant)
        participant.__str__ = mock.Mock(return_value="\tparticipant alice as Alice\n")

        box = Box("Test Box", [actor, participant])
        expect_str = """\tbox Test Box
\tActor John
\tparticipant alice as Alice
\tend\n"""
        self.assertEqual(str(box), expect_str)


class TestLink(unittest.TestCase):
    def setUp(self):
        self.source = mock.MagicMock(spec=Actor)
        self.source.id_ = "John"
        self.target = mock.MagicMock(spec=Participant)
        self.target.id_ = "Alice"

    def test_str_without_activation_deactivation(self):
        link = Link(self.source, self.target, "Solid-line", "Hello World")
        expected_str = "\tJohn->Alice: Hello World\n"
        self.assertEqual(str(link), expected_str)

    def test_str_with_arrow_type_classe(self):
        link = Link(self.source, self.target, ArrowTypes.SOLID_LINE, "Hello World")
        expected_str = "\tJohn->Alice: Hello World\n"
        self.assertEqual(str(link), expected_str)

    def test_str_with_activation(self):
        link = Link(
            self.source, self.target, "Solid-line", "Hello World", activate_target=True
        )
        expected_str = "\tJohn->Alice: Hello World\nactivate Alice\n"
        self.assertEqual(str(link), expected_str)

    def test_str_with_deactivation(self):
        link = Link(
            self.source,
            self.target,
            "Solid-line",
            "Hello World",
            deactivate_target=True,
        )
        expected_str = "\tJohn->Alice: Hello World\ndeactivate Alice\n"
        self.assertEqual(str(link), expected_str)


class TestNote(unittest.TestCase):
    def setUp(self) -> None:
        self.actor = mock.Mock(spec=Actor)
        self.actor.id_ = "actor"
        self.participant = mock.Mock(spec=Participant)
        self.participant.id_ = "participant"
        self.note = "This is a note"
        return super().setUp()

    def test_note_str_single_element(self) -> None:
        note = Note(self.note, self.actor)
        self.assertEqual(str(note), "\tNote over actor: This is a note\n")

    def test_note_str_multiple_elements(self) -> None:
        note_multiple = Note(
            self.note, [self.actor, self.participant], NotePosition.OVER
        )
        self.assertEqual(
            str(note_multiple), "\tNote over actor,participant: This is a note\n"
        )

    def test_note_position(self) -> None:
        note_left = Note(self.note, self.actor, NotePosition.LEFT_OF)
        self.assertEqual(str(note_left), "\tNote left of actor: This is a note\n")
        note_right = Note(self.note, self.actor, NotePosition.RIGHT_OF)
        self.assertEqual(str(note_right), "\tNote right of actor: This is a note\n")

    def test_note_assertion(self) -> None:
        with self.assertRaises(ValueError):
            Note(self.note, [self.actor, self.participant], NotePosition.LEFT_OF)
        with self.assertRaises(ValueError):
            Note(self.note, [self.actor, self.participant], NotePosition.RIGHT_OF)


class TestLoop(unittest.TestCase):
    def test_loop_str(self) -> None:
        link = mock.MagicMock(spec=Link)
        link.configure_mock(__str__=lambda _: "\tJohn->Alice: Hello World\n")
        loop = Loop("condition", [link])
        self.assertEqual(
            str(loop), "\tloop condition\n\tJohn->Alice: Hello World\n\tend\n"
        )


class TestAlt(unittest.TestCase):
    def setUp(self):
        self.link_1 = mock.MagicMock(spec=Link)
        self.link_1.configure_mock(__str__=lambda _: "\tA-->B: message\n")
        self.link_2 = mock.MagicMock(spec=Link)
        self.link_2.configure_mock(__str__=lambda _: "\tA-->B: message\n")

    def test_str_one_condition(self):
        alt = Alt({"condition": [self.link_1]})
        excepted_str = "\talt condition\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(alt), excepted_str)

    def test_str_multiple_condition(self):
        alt = Alt({"condition-1": [self.link_1], "condition-2": [self.link_2]})
        except_str = "\talt condition-1\n\tA-->B: message\n\telse condition-2\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(alt), except_str)


class TestOptional(unittest.TestCase):
    def setUp(self):
        self.link = mock.MagicMock(spec=Link)
        self.link.configure_mock(__str__=lambda _: "\tA-->B: message\n")

    def test_str_with_one_condition(self):
        optional = Optional("condition", [self.link])
        excepted_str = "\topt condition\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(optional), excepted_str)

    def test_str_with_multiple_condition(self):
        optional = Optional("condition-1", [self.link, self.link])
        excepted_str = "\topt condition-1\n\tA-->B: message\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(optional), excepted_str)


class TestParallel(unittest.TestCase):
    def setUp(self):
        self.link = mock.MagicMock(spec=Link)
        self.link.configure_mock(__str__=lambda _: "\tA-->B: message\n")

    def test_str_with_one_condition(self):
        parallel = Parallel({"condition": [self.link]})
        excepted_str = "\tpar condition\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(parallel), excepted_str)

    def test_str_with_multiple_condition(self):
        parallel = Parallel({"condition-1": [self.link], "condition-2": [self.link]})
        excepted_str = "\tpar condition-1\n\tA-->B: message\n\tand condition-2\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(parallel), excepted_str)

    def test_str_with_multiple_condition_and_logic(self):
        loop = mock.MagicMock(spec=Loop)
        loop.configure_mock(
            __str__=lambda _: "\tloop condition\n\tA-->B: message\n\tend\n"
        )
        parallel = Parallel({"condition-1": [self.link], "condition-2": [loop]})
        excepted_str = "\tpar condition-1\n\tA-->B: message\n\tand condition-2\n\tloop "
        excepted_str += "condition\n\tA-->B: message\n\tend\n\tend\n"
        self.assertEqual(str(parallel), excepted_str)


class TestCritical(unittest.TestCase):
    def setUp(self):
        self.link = mock.MagicMock(spec=Link)
        self.link.configure_mock(__str__=lambda _: "\tA-->B: message\n")

    def test_str_with_one_condition(self):
        critical = Critical("condition-1", [self.link], {"condition-2": [self.link]})
        expected_str = "\tcritical condition-1\n\tA-->B: message\n"
        expected_str += "\toption condition-2\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(critical), expected_str)

    def test_str_with_multiple_condition(self):
        critical = Critical(
            "condition-1",
            [self.link, self.link],
            {"condition-2": [self.link], "condition-3": [self.link]},
        )
        expected_str = "\tcritical condition-1\n\tA-->B: message\n"
        expected_str += "\tA-->B: message\n"
        expected_str += "\toption condition-2\n\tA-->B: message\n"
        expected_str += "\toption condition-3\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(critical), expected_str)


class TestBreak(unittest.TestCase):
    def setUp(self) -> None:
        self.link = mock.MagicMock(spec=Link)
        self.link.configure_mock(__str__=lambda _: "\tA-->B: message\n")

    def test_str_with_one_statement(self):
        break_ = Break("condition", [self.link])
        expected_str = "\tbreak condition\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(break_), expected_str)

    def test_str_with_multiple_statements(self):
        break_ = Break("condition", [self.link, self.link])
        expected_str = "\tbreak condition\n\tA-->B: message\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(break_), expected_str)


class TestRect(unittest.TestCase):
    def setUp(self) -> None:
        self.link = mock.MagicMock(spec=Link)
        self.link.configure_mock(__str__=lambda _: "\tA-->B: message\n")

    def test_str_with_one_statement(self):
        rect = Rect([self.link], (255, 0, 0))
        expected_str = "\trect rgb(255,0,0)\n\tA-->B: message\n\tend\n"
        self.assertEqual(str(rect), expected_str)

    def test_str_with_multiple_statements(self):
        rect = Rect([self.link, self.link], (255, 0, 0))
        expected_str = (
            "\trect rgb(255,0,0)\n\tA-->B: message\n\tA-->B: message\n\tend\n"
        )
        self.assertEqual(str(rect), expected_str)

    def test_rect_assertion(self):
        with self.assertRaises(ValueError):
            Rect([self.link, self.link], (257, 3, 4, 4))
        with self.assertRaises(ValueError):
            Rect([self.link, self.link], (257, 3, -4))


class TestSequenceDiagram(unittest.TestCase):
    def setUp(self) -> None:
        self.actor_1 = mock.MagicMock(spec=Actor)
        self.actor_1.configure_mock(__str__=lambda _: "\tactor A\n")
        self.actor_2 = mock.MagicMock(spec=Actor)
        self.actor_2.configure_mock(__str__=lambda _: "\tactor B\n")
        self.link_1 = mock.MagicMock(spec=Link)
        self.link_1.configure_mock(__str__=lambda _: "\tA-->B: message\n")
        self.link_2 = mock.MagicMock(spec=Link)
        self.link_2.configure_mock(__str__=lambda _: "\tA-->B: message\n")

    def test_str_without_auto_number(self):
        diagram = SequenceDiagram(
            "Test Diagram", [self.actor_1, self.actor_2, self.link_1, self.link_2]
        )
        expected_str = """---
title: Test Diagram
---
sequenceDiagram
\tactor A
\tactor B
\tA-->B: message
\tA-->B: message
"""
        self.assertEqual(diagram.script, expected_str)

    def test_str_with_auto_number(self):
        diagram = SequenceDiagram(
            "Test Diagram",
            [self.actor_1, self.actor_2, self.link_1, self.link_2],
            auto_number=True,
        )
        expected_str = """---
title: Test Diagram
---
sequenceDiagram
\tautonumber
\tactor A
\tactor B
\tA-->B: message
\tA-->B: message
"""
        self.assertEqual(diagram.script, expected_str)

    def test_str_with_config(self):
        config = Config(primary_color="red")
        diagram = SequenceDiagram(
            "Test Diagram",
            [self.actor_1, self.actor_2, self.link_1, self.link_2],
            config=config,
        )
        expected_str = """---
title: Test Diagram
---
%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t\t"primaryColor": "red"
\t\t}
\t}
}%%

sequenceDiagram
\tactor A
\tactor B
\tA-->B: message
\tA-->B: message
"""
        self.assertEqual(diagram.script, expected_str)



from typing import Optional as Option
from typing import Union

from mermaid.configuration import Config
from mermaid.graph import Graph
from mermaid.sequence.element import Actor, Box, Note, NotePosition, Participant, Rect
from mermaid.sequence.link import ArrowTypes, Link
from mermaid.sequence.logic import Alt, Break, Critical, Loop, Optional, Parallel


class SequenceDiagram(Graph):
    """SequenceDiagram class.

    This class represents a sequence diagram.

    Attributes:
        elements (list[Union[Actor, Participant, Box, Note, Link, Alt, Break, Critical, Loop, Optional, Parallel]]):
            The elements in the sequence diagram.
        auto_number (bool): Whether to automatically number the elements in the diagram.
        config (Config): The configuration for the sequence diagram.
    """

    def __init__(
        self,
        title: str,
        elements: list[
            Union[
                Actor,
                Participant,
                Box,
                Note,
                Link,
                Alt,
                Break,
                Critical,
                Loop,
                Optional,
                Parallel,
            ]
        ],
        auto_number: bool = False,
        config: Option[Config] = None,
    ) -> None:
        """Initialize a new SequenceDiagram.

        Args:
            title (str): The title of the sequence diagram.
            elements (list[Union[Actor, Participant, Box, Note, Link, Alt,
                                  Break, Critical, Loop, Optional, Parallel]]):
                The elements in the sequence diagram.
            auto_number (bool): Whether to automatically number the elements in the diagram. Defaults to False.
            config (Optional[Config]): The configuration for the sequence diagram. Defaults to None.
        """
        super().__init__(title, "", config)
        self.elements = elements
        self.auto_number = auto_number
        self._build_script()

    def _build_script(self) -> None:
        super()._build_script()
        script: str = "\nsequenceDiagram\n"
        if self.auto_number:
            script += "\tautonumber\n"
        for element in self.elements:
            script += str(element)
        self.script += script


__all__ = [
    "SequenceDiagram",
    "Actor",
    "Participant",
    "Box",
    "Note",
    "NotePosition",
    "Rect",
    "Link",
    "ArrowTypes",
    "Alt",
    "Break",
    "Critical",
    "Loop",
    "Optional",
    "Parallel",
]



"""UserJourney module.

This module provides the UserJourney class for creating and manipulating user journeys.
It also provides the Section and Task classes for representing sections and tasks in a user journey.

Classes:
    UserJourney: Represents a user's journey through a website or application.
    Section: Represents a section in a user's journey.
    Task: Represents a task in a section of a user's journey.
"""

from typing import Optional, Union

from mermaid.configuration import Config
from mermaid.graph import Graph

from .actor import Actor
from .section import Section
from .task import Task


class UserJourney(Graph):
    """UserJourney class.

    This class represents a user's journey through a website or application.

    Attributes:
        title (str): The title of the user's journey.
        sections (list[Union[Section, Task]]): The sections in the user's journey.
        config (Config): The configuration for the user's journey.
    """

    def __init__(
        self,
        title: str,
        sections: list[Union[Section, Task]],
        config: Optional[Config] = None,
    ) -> None:
        """Initialize a new UserJourney.

        Args:
            title (str): The title of the user's journey.
            sections (list[Union[Section, Task]]): The sections in the user's journey.
            config (Optional[Config]): The configuration for the user's journey. Defaults to None.
        """
        super().__init__(title, "", config)
        self.title: str = title
        self.sections: list[Union[Section, Task]] = sections
        self._build_script()

    def _build_script(self) -> None:
        """Build the script for the user's journey."""
        super()._build_script()
        script: str = f"\njourney\n\ttitle {self.title}\n"
        for section in self.sections:
            script += f"{section}\n"
        self.script += script


__all__ = [
    "UserJourney",
    "Section",
    "Task",
    "Actor",
]



from typing import Optional

from mermaid import text_to_snake_case

from .base import BaseTransition
from .state import End, Start, State


class Transition(BaseTransition):
    """Transition class.

    This class represents a transition in a state diagram.

    Attributes:
        from_state (State): The state from which the transition starts.
        to_state (State): The state to which the transition ends.
        label (str): The label of the transition.
    """

    def __init__(
        self, from_: Optional[State] = None, to: Optional[State] = None, label: str = ""
    ) -> None:
        """Initialize a new Transition.

        Args:
            from_ (State): The state from which the transition starts.
            to (State): The state to which the transition ends.
            label (str): The label of the transition.
        """
        if from_ is None:
            from_ = Start()
        if to is None:
            to = End()
        self.from_state: State = from_
        self.to_state: State = to
        self.label: str = label

    def __str__(self):
        """Return the string representation of the transition."""
        if not self.label:
            return f"{self.from_state.id_} --> {self.to_state.id_}"
        return f"{self.from_state.id_} --> {self.to_state.id_} : {self.label}"


class Choice(BaseTransition):
    """Choice class.

    This class represents a choice in a state diagram.

    Attributes:
        from_state (State): The state from which the choice starts.
        to_states (list[State]): The states to which the choice ends.
        conditions (list[str]): The conditions of the choice.
    """

    def __init__(
        self,
        id_: str,
        from_: Optional[State] = None,
        to: Optional[list[State]] = None,
        conditions: Optional[list[str]] = None,
    ) -> None:
        """Initialize a new Choice.

        Args:
            id_ (str): The ID of the choice.
            from_ (Optional[State]): The state from which the choice starts.
            to (Optional[list[State]]): The states to which the choice ends.
            conditions (Optional[list[str]]): The conditions of the choice.
        """
        self.id_: str = text_to_snake_case(id_)
        self.from_state: Optional[State] = from_
        self.to_states: list[State] = to if to is not None else []
        self.conditions: list[str] = conditions if conditions is not None else []

    def __str__(self) -> str:
        """Return the string representation of the choice."""
        string: str = f"state {self.id_} <<choice>>"

        if self.from_state:
            string += f"\n{self.from_state.id_} --> {self.id_}"

        for state in self.to_states:
            string += f"\n{self.id_} --> {state.id_}"
            index = self.to_states.index(state)
            # check if there is a condition for the transition
            if len(self.conditions) > index:
                string += f" : {self.conditions[index]}"

        return string


class Fork(BaseTransition):
    """Fork class.

    This class represents a fork in a state diagram.

    Attributes:
        from_state (State): The state from which the fork starts.
        to_states (list[State]): The states to which the fork ends.
    """

    def __init__(
        self, id_: str, from_: Optional[State] = None, to: Optional[list[State]] = None
    ) -> None:
        """Initialize a new Fork.

        Args:
            id_ (str): The ID of the fork.
            from_ (Optional[State]): The state from which the fork starts.
            to (Optional[list[State]]): The states to which the fork ends.
        """
        self.id_: str = text_to_snake_case(id_)
        self.from_state: Optional[State] = from_
        self.to_states: list[State] = to if to is not None else []

    def __str__(self) -> str:
        """Return the string representation of the fork."""
        string: str = f"state {self.id_} <<fork>>"

        if self.from_state:
            string += f"\n{self.from_state.id_} --> {self.id_}"

        for state in self.to_states:
            string += f"\n{self.id_} --> {state.id_}"

        return string


class Join(BaseTransition):
    """Join class.

    This class represents a join in a state diagram.

    Attributes:
        from_states (list[State]): The states from which the join starts.
        to_state (State): The state to which the join ends.
    """

    def __init__(
        self, id_: str, from_: Optional[list[State]] = None, to: Optional[State] = None
    ) -> None:
        """Initialize a new Join.

        Args:
            id_ (str): The ID of the join.
            from_ (Optional[list[State]]): The states from which the join starts.
            to (Optional[State]): The state to which the join ends.
        """
        self.id_: str = text_to_snake_case(id_)
        self.from_states: list[State] = from_ if from_ is not None else []
        self.to_state: Optional[State] = to

    def __str__(self) -> str:
        """Return the string representation of the join."""
        string: str = f"state {self.id_} <<join>>"

        for state in self.from_states:
            string += f"\n{state.id_} --> {self.id_}"

        if self.to_state:
            string += f"\n{self.id_} --> {self.to_state.id_}"

        return string



import unittest

from mermaid import Direction
from mermaid.configuration import Config
from mermaid.flowchart import FlowChart, Link, LinkHead, LinkShape, Node
from mermaid.flowchart.node import NodeShape
from mermaid.style import Style


class TestNode(unittest.TestCase):
    def test_creating_node_without_content(self):
        node: Node = Node("First Node")
        expect_id: str = "first_node"
        expect_content: str = "First Node"
        expect_shape: NodeShape = NodeShape("[", "]")
        self.assertEqual(expect_id, node.id_)
        self.assertEqual(expect_content, node.content)
        self.assertEqual(expect_shape, node.shape)

    def test_creating_node_with_content(self):
        node: Node = Node("First Node", "this is my content", "hexagon")
        expect_id: str = "first_node"
        expect_content: str = "this is my content"
        expect_shape: NodeShape = NodeShape("{{", "}}")
        self.assertEqual(expect_id, node.id_)
        self.assertEqual(expect_content, node.content)
        self.assertEqual(expect_shape, node.shape)

    def test_string_repr_for_node(self):
        node: Node = Node("First Node", "this is my content", "hexagon")
        expect_string: str = "first_node{{" + '"this is my content"' + "}}"
        self.assertEqual(expect_string, str(node))

    def test_string_repr_for_node_with_sub_nodes(self):
        node_1: Node = Node("First Node")
        node_2: Node = Node("Second Node")
        node: Node = Node("Main Node", sub_nodes=[node_1, node_2])
        expect_string: str = """subgraph main_node ["Main Node"]
\tdirection LR
\tfirst_node["First Node"]
\tsecond_node["Second Node"]
end"""
        self.assertEqual(expect_string, str(node))

    def test_string_node_with_href_type_deafult(self):
        node: Node = Node("Node Name", href="www.github.com")
        expect_string: str = """node_name["Node Name"]
click node_name "www.github.com" _blank"""

        self.assertEqual(expect_string, str(node))

    def test_string_node_without_href_type_deafult(self):
        node: Node = Node("Node Name", href="www.github.com", href_type="top")
        expect_string: str = """node_name["Node Name"]
click node_name "www.github.com" _top"""

        self.assertEqual(expect_string, str(node))

    def test_string_node_with_styles(self):
        styles = [
            Style(name="firstStyle", fill="red"),
            Style(name="secondStyle", stroke="green"),
        ]
        node: Node = Node("Node Name", styles=styles)
        expect_string: str = """node_name["Node Name"]
node_name:::firstStyle
node_name:::secondStyle"""
        self.assertEqual(expect_string, str(node))

    def test_string_sub_nodes_with_styles(self):
        styles = [
            Style(name="firstStyle", fill="red"),
            Style(name="secondStyle", stroke="green"),
        ]
        node_1: Node = Node("First Node")
        node_2: Node = Node("Second Node")
        node: Node = Node("Main Node", sub_nodes=[node_1, node_2], styles=styles)
        expect_string: str = """subgraph main_node ["Main Node"]
\tdirection LR
\tfirst_node["First Node"]
\tsecond_node["Second Node"]
end
main_node:::firstStyle
main_node:::secondStyle"""
        self.assertEqual(expect_string, str(node))

    def test_string_sub_nodes_with_non_default_direction(self):
        node_1: Node = Node("First Node")
        node_2: Node = Node("Second Node")
        node: Node = Node(
            "Main Node", sub_nodes=[node_1, node_2], direction=Direction.TOP_TO_BOTTOM
        )
        expect_string: str = """subgraph main_node ["Main Node"]
\tdirection TB
\tfirst_node["First Node"]
\tsecond_node["Second Node"]
end"""
        self.assertEqual(expect_string, str(node))


class TestLink(unittest.TestCase):
    def setUp(self) -> None:
        self.node_1: Node = Node("First Node")
        self.node_2: Node = Node("Second Node")
        return super().setUp()

    def test_str_link_without_message(self):
        link: Link = Link(self.node_1, self.node_2)
        expect_string: str = "first_node --> second_node"
        self.assertEqual(expect_string, str(link))

    def test_str_link_with_message(self):
        message: str = "this is my message"
        link: Link = Link(self.node_1, self.node_2, message=message)
        expect_string: str = f"first_node -->|{message}| second_node"
        self.assertEqual(expect_string, str(link))

    def test_str_link_with_no_default_value(self):
        message: str = "this is my message"
        link: Link = Link(
            self.node_1,
            self.node_2,
            shape="dotted",
            head_left="bullet",
            head_right="cross",
            message=message,
        )
        expect_string: str = f"first_node o-.-x|{message}| second_node"
        self.assertEqual(expect_string, str(link))

    def test_str_link_with_enum_shape(self):
        link: Link = Link(self.node_1, self.node_2, shape=LinkShape.DOTTED)
        expect_string: str = "first_node -.-> second_node"
        self.assertEqual(expect_string, str(link))

    def test_str_link_with_enum_link_head(self):
        link: Link = Link(
            self.node_1,
            self.node_2,
            head_left=LinkHead.BULLET,
            head_right=LinkHead.CROSS,
        )
        expect_string: str = "first_node o--x second_node"
        self.assertEqual(expect_string, str(link))


class TestFlowChart(unittest.TestCase):
    def test_make_flowchart_script_with_default_orientation(self):
        nodes = [Node("First Node"), Node("Second Node"), Node("Third Node")]
        links = [
            Link(nodes[0], nodes[1], head_left="cross"),
            Link(nodes[1], nodes[2], head_right="bullet"),
        ]
        flowchart: FlowChart = FlowChart("simple flowchart", nodes, links)
        expect_script: str = f"""---
title: simple flowchart
---
flowchart TB
\t{nodes[0]}
\t{nodes[1]}
\t{nodes[2]}
\t{links[0]}
\t{links[1]}
"""
        self.assertEqual(expect_script, flowchart.script)

    def test_make_flowchart_script_without_default_orientation(self):
        nodes = [Node("First Node"), Node("Second Node"), Node("Third Node")]
        links = [
            Link(nodes[0], nodes[1], head_left="cross"),
            Link(nodes[1], nodes[2], head_right="bullet"),
        ]
        flowchart: FlowChart = FlowChart(
            "simple flowchart", nodes, links, orientation="LR"
        )
        expect_script: str = f"""---
title: simple flowchart
---
flowchart LR
\t{nodes[0]}
\t{nodes[1]}
\t{nodes[2]}
\t{links[0]}
\t{links[1]}
"""
        self.assertEqual(expect_script, flowchart.script)

    def test_make_flowchart_script_with_enum_orientation(self):
        nodes = [Node("First Node"), Node("Second Node"), Node("Third Node")]

        links = [
            Link(nodes[0], nodes[1], head_left="cross"),
            Link(nodes[1], nodes[2], head_right="bullet"),
        ]
        flowchart: FlowChart = FlowChart(
            "simple flowchart", nodes, links, orientation=Direction.LEFT_TO_RIGHT
        )
        expect_script: str = f"""---
title: simple flowchart
---
flowchart LR
\t{nodes[0]}
\t{nodes[1]}
\t{nodes[2]}
\t{links[0]}
\t{links[1]}
"""
        self.assertEqual(expect_script, flowchart.script)

    def test_make_flowchart_script_with_styles(self):
        styles = [
            Style(name="firstStyle", fill="red"),
            Style(name="secondStyle", stroke="green"),
        ]
        nodes = [
            Node("First Node", styles=[styles[0]]),
            Node("Second Node", styles=[styles[1]]),
            Node("Third Node"),
        ]
        links = [
            Link(nodes[0], nodes[1], head_left="cross"),
            Link(nodes[1], nodes[2], head_right="bullet"),
        ]
        styles = list(set(styles))
        flowchart: FlowChart = FlowChart("simple flowchart", nodes, links)
        expect_script: str = f"""---
title: simple flowchart
---
flowchart TB
\t{styles[0]}
\t{styles[1]}
\t{nodes[0]}
\t{nodes[1]}
\t{nodes[2]}
\t{links[0]}
\t{links[1]}
"""
        self.assertEqual(expect_script, flowchart.script)

    def test_make_flowchart_script_with_duplicate_styles(self):
        styles = [
            Style(name="firstStyle", fill="red"),
            Style(name="secondStyle", stroke="green"),
        ]
        nodes = [
            Node("First Node", styles=styles),
            Node("Second Node", styles=styles),
            Node("Third Node"),
        ]
        links = [
            Link(nodes[0], nodes[1], head_left="cross"),
            Link(nodes[1], nodes[2], head_right="bullet"),
        ]
        styles = list(set(styles))
        flowchart: FlowChart = FlowChart("simple flowchart", nodes, links)
        expect_script: str = f"""---
title: simple flowchart
---
flowchart TB
\t{styles[0]}
\t{styles[1]}
\t{nodes[0]}
\t{nodes[1]}
\t{nodes[2]}
\t{links[0]}
\t{links[1]}
"""
        self.assertEqual(expect_script, flowchart.script)

    def test_str_flowchart_with_config(self):
        nodes = [Node("First Node"), Node("Second Node"), Node("Third Node")]
        links = [
            Link(nodes[0], nodes[1], head_left="cross"),
            Link(nodes[1], nodes[2], head_right="bullet"),
        ]
        config = Config(primary_color="red")
        flowchart: FlowChart = FlowChart(
            "simple flowchart", nodes, links, config=config
        )
        sub_string = """---
title: simple flowchart
---
%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t\t"primaryColor": "red"
\t\t}
\t}
}%%
"""
        expect_script: str = f"""{sub_string}
flowchart TB
\t{nodes[0]}
\t{nodes[1]}
\t{nodes[2]}
\t{links[0]}
\t{links[1]}
"""
        self.assertEqual(expect_script, flowchart.script)



from dataclasses import dataclass
from typing import Optional


@dataclass
class Style:
    """Style class.

    This class represents a style definition for a Mermaid diagram.

    Attributes:
        name (str): The name of the style definition.
        fill (str): The fill color of the style definition.
        color (str): The text color of the style definition.
        font_weight (str): The font weight of the style definition.
        stroke_width (str): The stroke width of the style definition.
        stroke (str): The stroke color of the style definition.
        other (str): Any other style definition separated by ,.
    """

    name: str
    fill: Optional[str] = None
    color: Optional[str] = None
    font_weight: Optional[str] = None
    stroke_width: Optional[str] = None
    stroke: Optional[str] = None
    other: Optional[str] = None

    def __str__(self) -> str:
        """Return the string representation of the style definition."""
        string: str = f"classDef {self.name} "
        style_paramaters = ",".join(
            [
                f"{style}:{value}"
                for style, value in [
                    ("fill", self.fill),
                    ("color", self.color),
                    ("font-weight", self.font_weight),
                    ("stroke-width", self.stroke_width),
                    ("stroke", self.stroke),
                ]
                if value is not None
            ]
        )
        string += style_paramaters
        if self.other:
            string += f",{self.other}" if style_paramaters else self.other
        return string

    def __hash__(self) -> int:
        return hash(self.name)



import unittest

from mermaid.configuration import Config
from mermaid.piechart import PieChart


class TestPieChart(unittest.TestCase):
    def test_piechart_without_showdata(self):
        data = {
            "Calcium": 42.96,
            "Potassium": 50.05,
            "Magnesium": 10.01,
            "Iron": 5,
        }
        pie = PieChart("simple pie chart", data)
        expect_str = """---
title: simple pie chart
---
pie
\t"Calcium" : 42.96
\t"Potassium" : 50.05
\t"Magnesium" : 10.01
\t"Iron" : 5
"""
        self.assertEqual(expect_str, pie.script)

    def test_piechart_with_showdata(self):
        data = {
            "Calcium": 42.96,
            "Potassium": 50.05,
            "Magnesium": 10.01,
            "Iron": 5,
        }
        pie = PieChart("simple pie chart", data, show_data=True)
        expect_str = """---
title: simple pie chart
---
pie showData
\t"Calcium" : 42.96
\t"Potassium" : 50.05
\t"Magnesium" : 10.01
\t"Iron" : 5
"""
        self.assertEqual(expect_str, pie.script)

    def test_piechart_with_config(self):
        data = {
            "Calcium": 42.96,
            "Potassium": 50.05,
            "Magnesium": 10.01,
            "Iron": 5,
        }
        config = Config(primary_color="red")
        pie = PieChart("simple pie chart", data, show_data=True, config=config)
        expect_str = """---
title: simple pie chart
---
%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t\t"primaryColor": "red"
\t\t}
\t}
}%%

pie showData
\t"Calcium" : 42.96
\t"Potassium" : 50.05
\t"Magnesium" : 10.01
\t"Iron" : 5
"""
        self.assertEqual(expect_str, pie.script)



from typing import Optional, Union


class Entity:
    """Entity class.

    This class represents an entity in an ER diagram.

    Attributes:
        name (str): The name of the entity.
        attributes (list[str]): The attributes of the entity.
    """

    def __init__(
        self, name: str, attributes: Optional[dict[str, Union[list[str], str]]] = None
    ) -> None:
        """Initialize a new Entity.

        Args:
            name (str): The name of the entity.
            attributes (Optional[dict[str, Union[list[str], str]]]): The attributes of the entity.
        """
        self.name: str = name
        self.attributes: dict[str, Union[list[str], str]] = (
            attributes if attributes is not None else {}
        )

    def __str__(self) -> str:
        string_attributes: str = self.build_attributes()
        string: str = "".join([self.name, "{\n", string_attributes, "}"])
        return string

    def update_attributes(self, attributes: dict[str, str]) -> None:
        """Update the attributes of the entity.

        Args:
            attributes (dict[str, Union[list[str], str]]): The new attributes of the entity.
        """
        self.attributes.update(attributes)

    def add_attribute(
        self,
        name: str,
        type_: str,
        constraint: Optional[str] = None,
        comment: Optional[str] = None,
    ) -> None:
        """Add an attribute to the entity.

        Args:
            name (str): The name of the attribute.
            attribute_def (Union[list[str], str]): The definition of the attribute.
        """
        attribute_def: list[str] = [type_]
        if constraint is not None:
            attribute_def.append(constraint)
        if comment is not None:
            attribute_def.append(comment)
        self.attributes[name] = attribute_def

    def build_attributes(self) -> str:
        """Build the string representation of the attributes.

        Returns:
            str: The string representation of the attributes.
        """
        string_attributes: str = ""
        for att_name, att_def in self.attributes.items():
            string: str = ""
            if isinstance(att_def, str):
                string = f"\t{att_def} {att_name}\n"
            else:
                if len(att_def) == 1:
                    string = f"{att_def[0]} {att_name}\n"
                elif len(att_def) == 2:
                    if att_def[1] in ["PK", "FK", "UK"]:
                        string = f"\t{att_def[0]} {att_name} {att_def[1]}\n"
                    else:
                        string = f'\t{att_def[0]} {att_name} "{att_def[1]}"\n'
                else:
                    string = f'\t{att_def[0]} {att_name} {att_def[1]} "{att_def[2]}"\n'
            string_attributes += string
        return string_attributes



"""Mindmap module.

This module provides the Mindmap class for creating and manipulating mindmap
diagrams.

Classes:
    Mindmap: Represents a mindmap diagram.
"""

from typing import Optional

from mermaid.configuration import Config
from mermaid.graph import Graph

from .level import Level, LevelShape


class Mindmap(Graph):
    """Mindmap class.

    This class represents a mindmap diagram.

    Args:
        title (str): The title of the mindmap.
        levels (Optional[list[Level]]): The levels of the mindmap. Defaults to None.
        shape (Optional[LevelShape]): The shape of the level. Defaults to None.
        config (Optional[Config]): The configuration for the mindmap. Defaults to None.
    """

    def __init__(
        self,
        title: str,
        levels: Optional[list[Level]] = None,
        shape: Optional[LevelShape] = None,
        config: Optional[Config] = None,
    ) -> None:
        """Initialize a new Mindmap.

        Args:
            title (str): The title of the mindmap.
            levels (Optional[list[Level]]): The levels of the mindmap. Defaults to None.
            shape (Optional[LevelShape]): The shape of the level. Defaults to None.
            config (Optional[Config]): The configuration for the mindmap. Defaults to None.
        """
        super().__init__(title, "", config)
        self.levels: list[Level] = levels if levels else []
        self.shape: LevelShape = shape if shape else LevelShape.DEFAULT
        self._build_script()

    def _build_script(self) -> None:
        super()._build_script()
        script: str = "\nmindmap"
        script += f"\n\t{self.shape.start}{self.title}{self.shape.end}"
        script += "\n"
        for level in self.levels:
            script += f"{level}"

        # script += '\n'
        self.script += script


__all__ = ["Mindmap", "Level", "LevelShape"]



import base64
import os
from enum import Enum
from pathlib import Path
from typing import Optional, Union
from urllib.parse import urlencode

import requests
from requests import Response

from .graph import Graph


class Position(Enum):
    """
    This class represents the position of the node in a Mermaid diagram.
    """

    LEFT = "left"
    RIGHT = "right"
    CENTER = "center"
    NONE = "none"


class Mermaid:
    """
    This class represents a Mermaid diagram.

    Attributes:
        _diagram (str): The base64 encoded string of the Mermaid diagram script.
        svg_response (Response): The response from the GET request to the Mermaid SVG API.
        img_response (Response): The response from the GET request to the Mermaid IMG API.
    """

    def __init__(
        self,
        graph: Union[Graph, str],
        width: Optional[int] = None,
        height: Optional[int] = None,
        scale: Optional[float] = None,
        position: Union[Position, str] = Position.NONE,
    ):
        """
        The constructor for the Mermaid class.

        Parameters:
            graph (Graph): The Mermaid diagram. It can be an instance of the Graph class or a string representing the script.
            width (Optional[int]): The width of the SVG image.
            height (Optional[int]): The height of the SVG image.
            scale (Optional[float]): The scale of the SVG image.
                Must be an float between 1 and 3, and one of height or width must be provided.
            position (Union[Position, str]): The position of the node in the Mermaid diagram.
        """
        if scale:
            assert 1 <= scale <= 3, "Scale must be between 1 and 3"
            assert any([width, height]), (
                "One or both of width and height must be provided"
            )

        self.__position: str = position if isinstance(position, str) else position.value
        self.__height = height if height else None
        self.__width = width if width else None
        self.__scale = scale if scale else None

        self._diagram = self._get_encoded_script(
            graph if isinstance(graph, str) else graph.script
        )
        self._make_request_to_mermaid()

    def _build_query_params(self, image_format: Optional[str] = None) -> str:
        """
        Build the query parameters for the Mermaid API request.

        Parameters:
            image_format (Optional[str]): The image format for /img endpoint
        """
        params = {
            param: value
            for param, value in [
                ("width", self.__width),
                ("height", self.__height),
                ("scale", self.__scale),
                ("format", image_format),
            ]
            if value
        }

        return urlencode(params, doseq=True)

    def set_position(self, position: Union[Position, str]) -> None:
        """
        Set the position of the node in the Mermaid diagram.

        Parameters:
            position (Union[Position, str]): The position of the node.
        """
        self.__position = position if isinstance(position, str) else position.value

    @staticmethod
    def _get_encoded_script(script: str) -> str:
        # CRITICAL FIX: Explicit UTF-8 encoding before base64
        script_bytes = script.encode("utf-8")

        # Use URL-safe base64 encoding (replaces + with -, / with _)
        encoded = base64.urlsafe_b64encode(script_bytes).decode("ascii")
        return encoded

    def _repr_html_(self) -> str:
        """
        Return the text of the SVG response.

        Returns:
            str: The text of the SVG response.
        """
        if self.__position == Position.NONE.value:
            return self.svg_response.text
        return (
            f'<div style="text-align:{self.__position}">{self.svg_response.text}</div>'
        )

    def _make_request_to_mermaid(self) -> None:
        """
        Make GET requests to the Mermaid SVG and IMG APIs using
        the base64 encoded string of the Mermaid diagram script.
        """
        mermaid_server_adress: str = os.getenv(
            "MERMAID_INK_SERVER", "https://mermaid.ink"
        )

        self.svg_response: Response = requests.get(
            mermaid_server_adress
            + "/svg/"
            + self._diagram
            + "?"
            + self._build_query_params()
        )
        self.img_response: Response = requests.get(
            mermaid_server_adress
            + "/img/"
            + self._diagram
            + "?"
            + self._build_query_params(image_format="png")
        )

    def to_svg(self, path: Union[str, Path]) -> None:
        """
        Write the SVG response text to a file.

        Parameters:
            path (Union[str, Path]): The path of the file to write to.
        """
        with open(path, "w", encoding="utf-8") as file:
            file.write(self.svg_response.text)

    def to_png(self, path: Union[str, Path]) -> None:
        """
        Write the IMG response content to a file.

        Parameters:
            path (Union[str, Path]): The path of the file to write to.
        """
        with open(path, "wb") as file:
            file.write(self.img_response.content)


try:
    from IPython import get_ipython

    if get_ipython() is not None:
        from IPython.core.magic import register_cell_magic
        from IPython.display import Image, display

        @register_cell_magic
        def mermaidjs(line, cell):
            options = line.strip().split()
            script: str = cell.strip()
            graph: Graph = Graph("mermaid diagram", script)
            mermaid: Mermaid = Mermaid(graph)
            if "--img" in options:
                display(Image(mermaid.img_response.content))
            else:
                display(Mermaid(graph))

        get_ipython().register_magic_function(mermaidjs, magic_kind="cell")
except ImportError:
    print(
        "Warning: IPython is not installed. Mermaidjs magic function is not available."
    )



import os
import unittest
from typing import List, Tuple

import mermaid as md
from mermaid.graph import Graph


class TestUtils(unittest.TestCase):
    def setUp(self) -> None:
        self.file_test: str = "./test-graph.mmd"
        with open(self.file_test, "w") as file:
            file.write("""graph TD;
    A-->B;
    A-->C;
    B-->D;
    C-->D;""")
        return super().setUp()

    def test_read_file_should_raise_error(self):
        with self.assertRaises(FileNotFoundError):
            md.load("./file.mmd")

    def test_read_file(self):
        graph: Graph = md.load(self.file_test)
        expect_title: str = "test-graph"
        expect_script: str = """graph TD;
    A-->B;
    A-->C;
    B-->D;
    C-->D;"""
        self.assertEqual(graph.title, expect_title)
        self.assertEqual(graph.script, expect_script)

    def test_convert_text_to_snake_case(self):
        test_cases: List[Tuple[str, str]] = [
            ("This is my Input", "this_is_my_input"),
            ("Some_Underscores_in_Input_", "some_underscores_in_input_"),
            (".Some.dots in Input.", ".some.dots_in_input."),
            ("Some-dashes-in Input", "some-dashes-in_input"),
            ("dots.and-dashes input", "dots.and-dashes_input"),
            ("other |chars*in input#", "other__chars_in_input_"),
        ]

        for input_id, expected_out in test_cases:
            out = md.text_to_snake_case(input_id)
            self.assertEqual(expected_out, out)

    def tearDown(self) -> None:
        if os.path.exists(self.file_test):
            os.remove(self.file_test)
        return super().tearDown()



"""Graph module.

This module provides the Graph base class for creating and manipulating
different types of diagrams such as flowcharts, ER diagrams, etc.

Classes:
    Graph: Represents a base class for different types of diagrams.
"""

from dataclasses import dataclass
from pathlib import Path
from typing import Optional, Union

from mermaid.configuration import Config


@dataclass
class Graph:
    """Graph base class.

    This class serves as a base for other classes representing different
    types of diagrams like `Flowchart`, `ERDiagram`, etc.

    Attributes:
        title (str): The title of the diagram.
        script (str): The main script to create the diagram.
        config (Optional[Config]): The configuration for the diagram.
    """

    title: str
    script: str
    config: Optional[Config] = None

    def save(self, path: Optional[Union[Path, str]] = None) -> None:
        """Save the diagram to a file.

        Args:
            path (Optional[Union[Path,str]]): The path to save the diagram. If not
                provided, the diagram will be saved in the current directory
                with the title as the filename.
        Raises:
            ValueError: If the file extension is not '.mmd' or '.mermaid'.
        """

        if path is None:
            path = Path(f"./{self.title}.mmd")
        if isinstance(path, str):
            path = Path(path)

        if path.suffix not in [".mmd", ".mermaid"]:
            raise ValueError("File extension must be '.mmd' or '.mermaid'")
        with open(path, "w") as file:
            file.write(self.script)

    def _build_script(self) -> None:
        script: str = f"---\ntitle: {self.title}\n---"
        if self.config:
            script += "\n" + str(self.config)
        script += self.script
        self.script = script


__all__ = ["Graph"]



import unittest

from mermaid import Config, Direction
from mermaid.statediagram import (
    Choice,
    Composite,
    Concurrent,
    End,
    Fork,
    Join,
    Start,
    State,
    StateDiagram,
    Transition,
)
from mermaid.style import Style


class TestState(unittest.TestCase):
    def test_simple_state(self):
        state: State = State("My State")
        expect_string: str = "my_state : My State"
        self.assertEqual(expect_string, str(state))

    def test_state_with_content(self):
        state: State = State("My State", "This is my content")
        expect_string: str = "my_state : This is my content"
        self.assertEqual(expect_string, str(state))

    def test_start_state(self):
        start: Start = Start()
        expect_string: str = "[*]"
        self.assertEqual(expect_string, str(start))

    def test_end_state(self):
        end: End = End()
        expect_string: str = "[*]"
        self.assertEqual(expect_string, str(end))

    def test_composite_state(self):
        composite: Composite = Composite("Main State", "This is my content")
        expect_string: str = "main_state : This is my content"
        self.assertEqual(expect_string, str(composite))

    def test_concurrent_state_with_style(self):
        styles = [Style(name="style1", fill="red"), Style(name="style2", color="blue")]
        state: State = State("My State", styles=styles)
        expect_string: str = "my_state : My State\nmy_state:::style1\nmy_state:::style2"
        self.assertEqual(expect_string, str(state))


class TestComposite(unittest.TestCase):
    def setUp(self) -> None:
        self.start: Start = Start()
        self.end: End = End()
        self.state_1: State = State("First State")
        self.state_2: State = State("Second State")
        self.transition_1: Transition = Transition(
            self.state_1, self.state_2, "This is my label"
        )
        self.transition_2: Transition = Transition(to=self.state_1)
        return super().setUp()

    def test_simple_composite_without_content(self):
        composite: Composite = Composite("Main State")
        expect_string: str = "main_state : Main State"
        self.assertEqual(expect_string, str(composite))

    def test_simple_composite_with_content(self):
        composite: Composite = Composite("Main State", "This is my content")
        expect_string: str = "main_state : This is my content"
        self.assertEqual(expect_string, str(composite))

    def test_state_with_sub_states(self):
        composite: Composite = Composite(
            "Main State",
            sub_states=[self.state_1, self.state_2],
            direction=Direction.LEFT_TO_RIGHT,
        )
        expect_string: str = """main_state : Main State
state main_state {
\tdirection LR
\tfirst_state : First State
\tsecond_state : Second State
}"""
        self.assertEqual(expect_string, str(composite))

    def test_state_with_sub_states_and_str_direction(self):
        composite: Composite = Composite(
            "Main State", sub_states=[self.state_1, self.state_2], direction="LR"
        )
        expect_string: str = """main_state : Main State
state main_state {
\tdirection LR
\tfirst_state : First State
\tsecond_state : Second State
}"""
        self.assertEqual(expect_string, str(composite))

    def test_composite_with_sub_states_and_transitions(self):
        composite: Composite = Composite(
            "Main State",
            sub_states=[self.state_1, self.state_2],
            transitions=[self.transition_1, self.transition_2],
        )
        expect_string: str = """main_state : Main State
state main_state {
\tfirst_state : First State
\tsecond_state : Second State
\tfirst_state --> second_state : This is my label
\t[*] --> first_state
}"""
        self.assertEqual(expect_string, str(composite))

    def test_composite_with_sub_states_and_styles(self):
        styles = [
            Style(name="style1", fill="red", font_weight="bold"),
            Style(name="style2", color="blue"),
        ]
        composite: Composite = Composite(
            "Main State", sub_states=[self.state_1, self.state_2], styles=styles
        )
        expect_string: str = """main_state : Main State
main_state:::style1
main_state:::style2
state main_state {
\tfirst_state : First State
\tsecond_state : Second State
}"""
        self.assertEqual(expect_string, str(composite))


class TestConcurrent(unittest.TestCase):
    def setUp(self) -> None:
        self.start: Start = Start()
        self.end: End = End()
        self.state_1: State = State("First State")
        self.state_2: State = State("Second State")
        self.state_3: State = State("Third State")
        self.transition_1: Transition = Transition(self.state_1, self.state_2)
        self.transition_2: Transition = Transition(to=self.state_1)
        return super().setUp()

    def test_simple_concurrent_without_content(self):
        concurrent: Concurrent = Concurrent("Main State")
        expect_string: str = "main_state : Main State"
        self.assertEqual(expect_string, str(concurrent))

    def test_simple_concurrent_with_content(self):
        concurrent: Concurrent = Concurrent("Main State", "This is my content")
        expect_string: str = "main_state : This is my content"
        self.assertEqual(expect_string, str(concurrent))

    def test_concurrent_with_sub_goups(self):
        groups = [
            ([self.state_1, self.state_2], [self.transition_1, self.transition_2]),
            ([self.state_1], [self.transition_2]),
        ]
        concurrent: Concurrent = Concurrent("Main State", sub_groups=groups)

        expect_string: str = """main_state : Main State
state main_state {
\tfirst_state : First State
\tsecond_state : Second State
\tfirst_state --> second_state
\t[*] --> first_state
\t--
\tfirst_state : First State
\t[*] --> first_state
}"""
        self.assertEqual(expect_string, str(concurrent))

    def test_concurrent_with_sub_goups_and_styles(self):
        styles = [
            Style(name="style1", fill="red", font_weight="bold"),
            Style(name="style2", color="blue"),
        ]
        groups = [
            ([self.state_1, self.state_2], [self.transition_1, self.transition_2]),
            ([self.state_1], [self.transition_2]),
        ]
        concurrent: Concurrent = Concurrent(
            "Main State", sub_groups=groups, styles=styles
        )

        expect_string: str = """main_state : Main State
main_state:::style1
main_state:::style2
state main_state {
\tfirst_state : First State
\tsecond_state : Second State
\tfirst_state --> second_state
\t[*] --> first_state
\t--
\tfirst_state : First State
\t[*] --> first_state
}"""

        self.assertEqual(expect_string, str(concurrent))


class TestTransition(unittest.TestCase):
    def setUp(self) -> None:
        self.start: Start = Start()
        self.end: End = End()
        self.state_1: State = State("First State")
        self.state_2: State = State("Second State")
        return super().setUp()

    def test_simple_transition(self):
        transition: Transition = Transition(
            self.state_1, self.state_2, "This is my label"
        )
        expect_string: str = "first_state --> second_state : This is my label"
        self.assertEqual(expect_string, str(transition))

    def test_transition_from_start_to_state(self):
        transition: Transition = Transition(to=self.state_1)
        expect_string: str = "[*] --> first_state"
        self.assertEqual(expect_string, str(transition))

    def test_transition_from_state_to_end(self):
        transition: Transition = Transition(from_=self.state_1)
        expect_string: str = "first_state --> [*]"
        self.assertEqual(expect_string, str(transition))


class TestChoice(unittest.TestCase):
    def setUp(self) -> None:
        self.start: Start = Start()
        self.end: End = End()
        self.state_1: State = State("First State")
        self.state_2: State = State("Second State")
        self.state_3: State = State("Third State")
        return super().setUp()

    def test_withot_to_and_from_state(self):
        choice: Choice = Choice("My Choice")
        expect_string: str = "state my_choice <<choice>>"
        self.assertEqual(expect_string, str(choice))

    def test_without_to_and_with_from_state(self):
        choice: Choice = Choice("My Choice", self.state_1)
        expect_string: str = "state my_choice <<choice>>\nfirst_state --> my_choice"
        self.assertEqual(expect_string, str(choice))

    def test_with_to_and_without_from_state(self):
        choice: Choice = Choice("My Choice", to=[self.state_1, self.state_2])
        expect_string: str = """state my_choice <<choice>>
my_choice --> first_state
my_choice --> second_state"""
        self.assertEqual(expect_string, str(choice))

    def test_with_to_and_from_state(self):
        choice: Choice = Choice("My Choice", self.state_1, [self.state_2, self.state_3])
        expect_string: str = """state my_choice <<choice>>
first_state --> my_choice
my_choice --> second_state
my_choice --> third_state"""
        self.assertEqual(expect_string, str(choice))

    def test_with_to_and_from_state_with_conditions(self):
        choice: Choice = Choice(
            "My Choice",
            self.state_1,
            [self.state_2, self.state_3],
            ["condition 1", "condition 2"],
        )
        expect_string: str = """state my_choice <<choice>>
first_state --> my_choice
my_choice --> second_state : condition 1
my_choice --> third_state : condition 2"""
        self.assertEqual(expect_string, str(choice))


class TestFork(unittest.TestCase):
    def setUp(self) -> None:
        self.start: Start = Start()
        self.end: End = End()
        self.state_1: State = State("First State")
        self.state_2: State = State("Second State")
        self.state_3: State = State("Third State")
        return super().setUp()

    def test_without_to_and_from_state(self):
        fork: Fork = Fork("My Fork")
        expect_string: str = "state my_fork <<fork>>"
        self.assertEqual(expect_string, str(fork))

    def test_without_to_and_with_from_state(self):
        fork: Fork = Fork("My Fork", self.state_1)
        expect_string: str = "state my_fork <<fork>>\nfirst_state --> my_fork"
        self.assertEqual(expect_string, str(fork))

    def test_with_to_and_without_from_state(self):
        fork: Fork = Fork("My Fork", to=[self.state_1, self.state_2])
        expect_string: str = """state my_fork <<fork>>
my_fork --> first_state
my_fork --> second_state"""
        self.assertEqual(expect_string, str(fork))

    def test_with_to_and_from_state(self):
        fork: Fork = Fork("My Fork", self.state_1, [self.state_2, self.state_3])
        expect_string: str = """state my_fork <<fork>>
first_state --> my_fork
my_fork --> second_state
my_fork --> third_state"""
        self.assertEqual(expect_string, str(fork))


class TestJoin(unittest.TestCase):
    def setUp(self) -> None:
        self.start: Start = Start()
        self.end: End = End()
        self.state_1: State = State("First State")
        self.state_2: State = State("Second State")
        self.state_3: State = State("Third State")
        return super().setUp()

    def test_without_to_and_from_state(self):
        join: Join = Join("My Join")
        expect_string: str = "state my_join <<join>>"
        self.assertEqual(expect_string, str(join))

    def test_without_to_and_with_from_state(self):
        join: Join = Join("My Join", [self.state_1])
        expect_string: str = "state my_join <<join>>\nfirst_state --> my_join"
        self.assertEqual(expect_string, str(join))

    def test_with_to_and_without_from_state(self):
        join: Join = Join("My Join", to=self.state_1)
        expect_string: str = """state my_join <<join>>
my_join --> first_state"""
        self.assertEqual(expect_string, str(join))

    def test_with_to_and_from_state(self):
        join: Join = Join("My Join", [self.state_1, self.state_2], self.state_3)
        expect_string: str = """state my_join <<join>>
first_state --> my_join
second_state --> my_join
my_join --> third_state"""
        self.assertEqual(expect_string, str(join))


class TestStateDiagram(unittest.TestCase):
    def setUp(self) -> None:
        self.start: Start = Start()
        self.end: End = End()
        self.state_1: State = State("First State")
        self.state_2: State = State("Second State")
        self.state_3: State = State("Third State")
        self.transition_1: Transition = Transition(
            self.state_1, self.state_2, "This is my label"
        )
        self.transition_2: Transition = Transition(to=self.state_1)
        self.choice: Choice = Choice(
            "My Choice",
            self.state_1,
            [self.state_2, self.state_3],
            ["condition 1", "condition 2"],
        )
        self.fork: Fork = Fork("My Fork", self.state_1, [self.state_2, self.state_3])
        self.join: Join = Join("My Join", [self.state_1, self.state_2], self.state_3)
        self.composite: Composite = Composite(
            "Main State",
            sub_states=[self.state_1, self.state_2],
            transitions=[self.transition_1, self.transition_2],
        )
        self.concurrent: Concurrent = Concurrent(
            "Main State",
            sub_groups=[
                ([self.state_1, self.state_2], [self.transition_1, self.transition_2]),
                ([self.state_1], [self.transition_2]),
            ],
        )

        return super().setUp()

    def test_state_diagram_version(self):
        list_verions = [
            StateDiagram("My State Diagram", [], [], "v1"),
            StateDiagram("My State Diagram", [], [], "v2"),
        ]
        list_expect = ["stateDiagram", "stateDiagram-v2"]

        def get_expected_script(version):
            return f"""---
title: My State Diagram
---
{version}
"""

        for i, version in enumerate(list_verions):
            expect_string = get_expected_script(list_expect[i])
            self.assertEqual(expect_string, version.script)

    def test_state_diagram_with_states(self):
        state_diagram: StateDiagram = StateDiagram(
            "My State Diagram", [self.state_1, self.state_2], []
        )
        expect_string: str = f"""---
title: My State Diagram
---
stateDiagram-v2
\t{self.state_1}
\t{self.state_2}
"""
        self.assertEqual(expect_string, state_diagram.script)

    def test_state_diagram_with_transitions(self):
        state_diagram: StateDiagram = StateDiagram(
            "My State Diagram",
            [self.state_1, self.state_2],
            [self.transition_1, self.transition_2],
        )
        expect_string: str = f"""---
title: My State Diagram
---
stateDiagram-v2
\t{self.state_1}
\t{self.state_2}
\t{self.transition_1}
\t{self.transition_2}
"""
        self.assertEqual(expect_string, state_diagram.script)

    def test_state_diagram_with_composite(self):
        state_diagram: StateDiagram = StateDiagram("My State Diagram", [self.composite])
        expect_string: str = f"""---
title: My State Diagram
---
stateDiagram-v2
\t{self.composite}
"""
        self.assertEqual(expect_string, state_diagram.script)

    def test_state_diagram_with_concurrent(self):
        transition = Transition(from_=self.concurrent, label="This is my label")
        state_diagram: StateDiagram = StateDiagram(
            "My State Diagram", [self.concurrent], [transition]
        )
        expect_string: str = f"""---
title: My State Diagram
---
stateDiagram-v2
\t{self.concurrent}
\t{transition}
"""
        self.assertEqual(expect_string, state_diagram.script)

    def test_state_diagram_with_styles(self):
        styles = [
            Style(name="style1", fill="red", font_weight="bold"),
            Style(name="style2", color="blue"),
        ]

        states = [
            State("First State", styles=[styles[0]]),
            State("Second State", styles=[styles[1]]),
        ]
        styles = list(set(styles))
        state_diagram: StateDiagram = StateDiagram("My State Diagram", states)
        expect_string: str = f"""---
title: My State Diagram
---
stateDiagram-v2
\t{styles[0]}
\t{styles[1]}
\t{states[0]}
\t{states[1]}
"""
        self.assertEqual(expect_string, state_diagram.script)

    def test_state_diagram_with_duplicate_styles(self):
        styles = [
            Style(name="style1", fill="red", font_weight="bold"),
            Style(name="style1", fill="red", font_weight="bold"),
        ]

        states = [
            State("First State", styles=[styles[0]]),
            State("Second State", styles=[styles[1]]),
        ]
        styles = list(set(styles))
        state_diagram: StateDiagram = StateDiagram("My State Diagram", states)
        expect_string: str = f"""---
title: My State Diagram
---
stateDiagram-v2
\t{styles[0]}
\t{states[0]}
\t{states[1]}
"""
        self.assertEqual(expect_string, state_diagram.script)

    def test_state_diagram_with_direction(self):
        state_diagram: StateDiagram = StateDiagram(
            "My State Diagram",
            [self.state_1, self.state_2],
            direction=Direction.LEFT_TO_RIGHT,
        )
        expect_string: str = f"""---
title: My State Diagram
---
stateDiagram-v2
\tdirection LR
\t{self.state_1}
\t{self.state_2}
"""
        self.assertEqual(expect_string, state_diagram.script)

    def test_state_diagram_with_config(self):
        config = Config(primary_color="red")
        state_diagram: StateDiagram = StateDiagram(
            "My State Diagram", [self.state_1, self.state_2], config=config
        )
        sub_string = """---
title: My State Diagram
---
%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t\t"primaryColor": "red"
\t\t}
\t}
}%%
"""
        except_str = f"""{sub_string}
stateDiagram-v2
\t{self.state_1}
\t{self.state_2}
"""
        self.assertEqual(except_str, state_diagram.script)



"""Module for the Level class

This module contains the Level class which represents a level in a mindmap.

Example:
    >>> # Create a level with the name 'Level 1' and a circle shape
    >>> Level('Level 1', LevelShape.CIRCLE)])
    >>> # Create a level with the name 'Level 1' and a circle shape with an icon
    >>> Level('Level 1', LevelShape.CIRCLE, Icon('fa', 'fa-book'))
"""

from enum import Enum
from typing import Optional

from mermaid import text_to_snake_case
from mermaid.icon import Icon


class LevelShape(Enum):
    """Enum for the shape of a level

    Args:
        start (str): The start of the shape
        end (str): The end of the shape
    """

    SQUARE = ("[", "]")
    ROUNDED_SQUARE = ("(", ")")
    CIRCLE = ("((", "))")
    BANG = ("))", "((")
    CLOUD = (")", "(")
    HEXAGON = ("{{", "}}")
    DEFAULT = ("", "")

    def __init__(self, start: str, end: str) -> None:
        """Constructor for the LevelShape class

        Args:
            start (str): the start of the shape.
            end (str): the end of the shape.
        """
        self.end: str = end
        self.start: str = start


class Level:
    """Class for a level in a mindmap"""

    def __init__(
        self,
        name: str,
        children: Optional[list["Level"]] = None,
        shape: Optional[LevelShape] = None,
        icon: Optional[Icon] = None,
    ) -> None:
        """Constructor for the Level class

        Args:
            name (str): The name of the level
            children (Optional[list[Level]], optional): The children of the level. Defaults to None.
            shape (Optional[LevelShape], optional): The shape of the level. Defaults to None.
            icon (Optional[Icon], optional): The icon of the level. Defaults to None.

        example:
        >>> # Create a level with the name 'Level 1' and a circle shape
        >>> Level('Level 1', LevelShape.CIRCLE)])
        >>> # Create a level with the name 'Level 1' and a circle shape with an icon
        >>> Level('Level 1', LevelShape.CIRCLE, Icon('fa', 'fa-book'))

        """
        self.id_: str = text_to_snake_case(name)
        self.name: str = name
        self.children: list["Level"] = children if children else []
        self.shape: LevelShape = shape if shape else LevelShape.DEFAULT
        self.icon: Optional[Icon] = icon

    def add_child(self, child: "Level") -> None:
        """Add a child to the level

        Args:
            child (Level): The child to add
        """
        self.children.append(child)

    def __str__(self) -> str:
        """String representation of the level

        Returns:
            str: The string representation of the level
        """
        return self.list_str_to_str(self.list_str())

    def list_str(self) -> list:
        """List representation of the level

        Returns:
            list: The list representation of the level
        """
        string: str = f"{self.shape.start}{self.name}{self.shape.end}"
        if not len(self.children):
            if self.icon:
                return [string, f"::icon({self.icon.type_} {self.icon.name})"]
            return [string]

        list_str: list = []
        for child in self.children:
            list_str += child.list_str()
        return [string, list_str]

    def list_str_to_str(self, list_str: list, depth: int = 2) -> str:
        """Convert a list representation of a level to a string

        Args:
            list_str (list): The list representation of the level
            depth (int, optional): The depth of the level. Defaults to 2.

        Returns:
            str: [description]
        """
        result: str = ""
        for item in list_str:
            if isinstance(item, list):
                result += self.list_str_to_str(item, depth + 1)
            else:
                result += "\t" * depth + item + "\n"
        return result



from enum import Enum
from typing import Union

from .node import Node

LINK_SHAPES: dict[str, str] = {
    "normal": "--",
    "dotted": "-.-",
    "thick": "==",
    "hidden": "~~~",
}

LINK_HEADS: dict[str, str] = {
    "none": "",
    "arrow": ">",
    "left-arrow": "<",
    "bullet": "o",
    "cross": "x",
}


class LinkShape(Enum):
    """LinkShape Enum.

    This enum represents the different shapes a link can have in a flowchart.
    """

    NORMAL = "normal"
    DOTTED = "dotted"
    THICK = "thick"
    HIDDEN = "hidden"


class LinkHead(Enum):
    """LinkHead Enum.

    This enum represents the different types of heads a link can have in a flowchart.
    """

    NONE = "none"
    ARROW = "arrow"
    LEFT_ARROW = "left-arrow"
    BULLET = "bullet"
    CROSS = "cross"


class Link:
    """Link class.

    This class represents a link between nodes in a flowchart.

    Attributes:
        origin (Node): The origin node of the link.
        end (Node): The end node of the link.
        shape (Union[str, LinkShape]): The shape of the link.
        head_left (Union[str, LinkHead]): The head of the link at the origin node.
        head_right (Union[str, LinkHead]): The head of the link at the end node.
        message (str): The message of the link.
    """

    def __init__(
        self,
        origin: Node,
        end: Node,
        shape: Union[str, LinkShape] = "normal",
        head_left: Union[str, LinkHead] = "none",
        head_right: Union[str, LinkHead] = "arrow",
        message: str = "",
    ) -> None:
        """Initialize a new Link.

        Args:
            origin (Node): The origin node of the link.
            end (Node): The end node of the link.
            shape (Union[str, LinkShape]): The shape of the link.
            head_left (Union[str, LinkHead]): The head of the link at the origin node.
            head_right (Union[str, LinkHead]): The head of the link at the end node.
            message (str): The message of the link.
        """
        self.origin: Node = origin
        self.end: Node = end
        self.head_left: str = LINK_HEADS[
            head_left if isinstance(head_left, str) else head_left.value
        ]
        self.head_right: str = LINK_HEADS[
            head_right if isinstance(head_right, str) else head_right.value
        ]
        self.shape: str = LINK_SHAPES[shape if isinstance(shape, str) else shape.value]
        self.message: str = f"|{message}|" if message else message

    def __str__(self) -> str:
        """Return a string representation of the link.

        If the link has a message, it includes the message in the string representation.
        Otherwise, it returns a string representation of a link without a message.

        Returns:
            str: A string representation of the link.
        """
        if self.message:
            element: list[str] = [
                self.origin.id_,
                " ",
                self.head_left,
                self.shape,
                self.head_right,
                self.message,
                " ",
                self.end.id_,
            ]
        else:
            element = [
                self.origin.id_,
                " ",
                self.head_left,
                self.shape,
                self.head_right,
                " ",
                self.end.id_,
            ]
        return "".join(element)



from enum import Enum
from typing import Union

from mermaid.sequence.element import Actor, Participant

ARROW_TYPES: dict[str, str] = {
    "Solid-line": "->",
    "Dotted-line": "-->",
    "Solid-arrow": "->>",
    "Dotted-arrow": "-->>",
    "Solid-cross": "-x",
    "Dotted-cross": "--x",
    "Solid-async": "-)",
    "Dotted-async": "--)",
}


class ArrowTypes(Enum):
    """Arrow types for mermaid sequence diagram."""

    SOLID_LINE = "->"
    DOTTED_LINE = "-->"
    SOLID_ARROW = "->>"
    DOTTED_ARROW = "-->>"
    SOLID_CROSS = "-x"
    DOTTED_CROSS = "--x"
    SOLID_ASYNC = "-)"
    DOTTED_ASYNC = "--)"


class Link:
    """Link class for mermaid sequence diagram.

    Args:
        source (ActorParticipant): Source of the link.
        target (ActorParticipant): Target of the link.
        type_ (Union[str,ArrowTypes]): Type of the link.
        message (str): Message to be displayed over the link.
        activate_target (bool, optional): Activate the target. Defaults to False.
        deactivate_target (bool, optional): Deactivate the target. Defaults to False.

    """

    def __init__(
        self,
        source: Union[Actor, Participant],
        target: Union[Actor, Participant],
        type_: Union[str, ArrowTypes],
        message: str,
        activate_target: bool = False,
        deactivate_target: bool = False,
    ) -> None:
        self.source: Union[Actor, Participant] = source
        self.target: Union[Actor, Participant] = target
        self.type_: Union[str, ArrowTypes] = (
            ARROW_TYPES[type_] if isinstance(type_, str) else type_.value
        )
        self.activate_target: bool = activate_target
        self.deactivate_target: bool = deactivate_target
        self.message: str = message

    def __str__(self) -> str:
        string: str = (
            f"\t{self.source.id_}{self.type_}{self.target.id_}: {self.message}\n"
        )
        if self.activate_target:
            string += f"activate {self.target.id_}\n"
        if self.deactivate_target:
            string += f"deactivate {self.target.id_}\n"

        return string



from typing import Union

from .actor import Actor


class Task:
    """Task class.

    This class represents a task in a section of a user's journey.

    Attributes:
        name (str): The name of the task.
    """

    def __init__(
        self, name: str, score: int, actors: Union[list[Actor], Actor]
    ) -> None:
        """Initialize a new Task.

        Args:
            name (str): The name of the task.
        """
        self.name: str = name
        self.score: int = score
        self.actors: list[Actor] = actors if isinstance(actors, list) else [actors]

    def __str__(self) -> str:
        """Return a string representation of the task.

        Returns:
            str: A string representation of the task.
        """
        actor_string: str = ", ".join([actor.name for actor in self.actors])
        return f"\t\t{self.name}: {self.score} : {actor_string}"



from typing import Union

from .element import Element
from .requirement import Requirement


class Link:
    """Link class.

    This class represents a link between elements or requirements in a requirement diagram.

    Attributes:
        source (Union[Element, Requirement]): The source of the link.
        destination (Union[Element, Requirement]): The destination of the link.
        type_ (str): The type of the link.
    """

    def __init__(
        self,
        source: Union[Element, Requirement],
        destination: Union[Element, Requirement],
        type_: str,
    ) -> None:
        """Initialize a new Link.

        Args:
            source (Union[Element, Requirement]): The source of the link.
            destination (Union[Element, Requirement]): The destination of the link.
            type_ (str): The type of the link.
        """
        self.source: Union[Element, Requirement] = source
        self.destination: Union[Element, Requirement] = destination
        self.type_: str = type_

    def __str__(self) -> str:
        """Return a string representation of the link.

        Returns:
            str: A string representation of the link.
        """
        return f"{self.source.name} - {self.type_} -> {self.destination.name}"



import unittest

from mermaid.style import Style


class TestStyle(unittest.TestCase):
    def test_str_style_with_just_name(self):
        style = Style(name="style1")
        self.assertEqual(str(style), "classDef style1 ")

    def test_str_style_with_some_style(self):
        style = Style(name="style1", fill="red", color="white")
        self.assertEqual(str(style), "classDef style1 fill:red,color:white")

    def test_str_style_with_other_style(self):
        style = Style(name="style1", other="fill:blue,stroke:yellow")
        self.assertEqual(str(style), "classDef style1 fill:blue,stroke:yellow")

    def test_str_style_with_defined_and_other_style(self):
        style = Style(name="style1", fill="red", other="stroke:#333,stroke-width:4px")
        self.assertEqual(
            str(style), "classDef style1 fill:red,stroke:#333,stroke-width:4px"
        )



import unittest

from mermaid import Mermaid
from mermaid.graph import Graph


class TestMermaidWithMalayalam(unittest.TestCase):
    def setUp(self) -> None:
        self.script: str = """
graph TD
    A["കേരളം"] --> B["ഇന്ത്യയുടെ സംസ്ഥാനം"]
    A --> C["തെക്കുപടിഞ്ഞാറേ അറ്റത്തുള്ള സംസ്ഥാനം"]
"""
        self.name: str = "malayalam-graph"
        self.graph: Graph = Graph(self.name, self.script)
        self.mermaid_object = Mermaid(self.graph)

    def test_make_request_to_mermaid_api_for_svg(self):
        self.assertTrue(self.mermaid_object.svg_response.status_code == 200)

    def test_make_request_to_mermaid_api_for_png(self):
        self.assertTrue(self.mermaid_object.img_response.status_code == 200)



import os
import unittest
from pathlib import Path

from mermaid.graph import Graph


class TestGraph(unittest.TestCase):
    def setUp(self) -> None:
        script: str = """graph TD;
    A-->B;
    A-->C;
    B-->D;
    C-->D;"""
        self.graph_test: Graph = Graph("test-graph", script)
        return super().setUp()

    def test_save_graph_without_path(self) -> None:
        self.graph_test.save()
        self.assertTrue(Path.exists(Path("./test-graph.mmd")))

    def test_save_graph_with_path(self) -> None:
        self.graph_test.save(Path("./test-graph-file.mmd"))
        self.assertTrue(Path.exists(Path("./test-graph-file.mmd")))

    def test_save_graph_with_path_str(self) -> None:
        self.graph_test.save("./test-graph-str-path.mmd")
        self.assertTrue(Path.exists(Path("./test-graph-str-path.mmd")))

    def test_save_graph_name_contain_space(self) -> None:
        self.graph_test.title = "test graph file"
        self.graph_test.save(Path("./test graph file.mmd"))
        self.assertTrue(Path.exists(Path("./test graph file.mmd")))

    def test_save_should_raise_valueerror(self):
        self.graph_test.title = "file-name"
        with self.assertRaises(ValueError):
            self.graph_test.save(Path("./file-name.txt"))

    def tearDown(self) -> None:
        if os.path.exists("./test-graph.mmd"):
            os.remove("./test-graph.mmd")
        if os.path.exists("./test-graph-file.mmd"):
            os.remove("./test-graph-file.mmd")
        if os.path.exists("./test graph file.mmd"):
            os.remove("./test graph file.mmd")
        if os.path.exists("./test-graph-str-path.mmd"):
            os.remove("./test-graph-str-path.mmd")
        return super().tearDown()



"""Mermaid module.

This module provides the Mermaid class for creating and manipulating Mermaid diagrams.
It also provides utility functions for loading data and converting text to snake case.

Classes:
    Mermaid: Represents a Mermaid diagram.

Functions:
    load(file_path): Load data from a file.
    text_to_snake_case(text): Convert a string of text to snake case.
"""

from enum import Enum

from .__main__ import Mermaid, Position
from .configuration import Config
from .graph import Graph
from .icon import Icon
from .style import Style
from .utils import load, text_to_snake_case

__version__: str = "0.8.3"


class Direction(Enum):
    """Enum for representing the direction of a Mermaid diagram."""

    LEFT_TO_RIGHT = "LR"
    RIGHT_TO_LEFT = "RL"
    TOP_TO_BOTTOM = "TB"
    BOTTOM_TO_TOP = "BT"


__all__ = [
    "Mermaid",
    "load",
    "Direction",
    "Graph",
    "Style",
    "Config",
    "Icon",
    "Position",
    "text_to_snake_case",
]



"""State module.

This module contains the classes to represent the states in a state diagram.
"""

from typing import Optional, Union

from mermaid import Direction, text_to_snake_case
from mermaid.style import Style

from .base import BaseTransition


class State:
    """State class.

    This class represents a state in a state diagram.

    Attributes:
        id_ (str): The id of the state.
        content (str): The content of the state.
        styles (list[Style]): The styles of the state.
    """

    def __init__(
        self, id_: str, content: str = "", styles: Optional[list[Style]] = None
    ) -> None:
        """Initialize a new State.

        Args:
            id_ (str): The id of the state.
            content (str): The content of the state.
            styles (Optional[list[Style]]): The styles of the state.
        """
        self.id_: str = text_to_snake_case(id_)
        self.content: str = content if content else id_
        self.styles: list[Style] = styles if styles is not None else []

    def __str__(self) -> str:
        """Return the string representation of the state."""
        string: str = f"{self.id_} : {self.content}"
        for style in self.styles:
            string += f"\n{self.id_}:::{style.name}"
        return string


class Start(State):
    """Start class.

    This class represents the start state in a state diagram.
    """

    def __init__(self) -> None:
        super().__init__(id_="[*]")
        self.id_: str = "[*]"

    def __str__(self) -> str:
        return f"{self.id_}"


class End(State):
    """End class.

    This class represents the end state in a state diagram.
    """

    def __init__(self) -> None:
        super().__init__(id_="[*]")
        self.id_: str = "[*]"

    def __str__(self) -> str:
        return f"{self.id_}"


class Composite(State):
    """Composite class.

    This class represents a composite state in a state diagram.

    Attributes:
        sub_states (list[State]): The sub states of the composite state.
        transitions (list[BaseTransition]): The transitions of the composite state.
        direction (Union[str,Direction]): The direction of the composite state.
        styles (list[Style]): The styles of the composite state.
    """

    def __init__(
        self,
        id_: str,
        content: str = "",
        sub_states: Optional[list[State]] = None,
        transitions: Optional[list[BaseTransition]] = None,
        direction: Optional[Union[str, Direction]] = None,
        styles: Optional[list[Style]] = None,
    ) -> None:
        """Initialize a new Composite.

        Args:
            id_ (str): The id of the state.
            content (str): The content of the state.
            sub_states (Optional[list[State]]): The sub states of the composite state.
            transitions (Optional[list[BaseTransition]]): The transitions of the composite state.
            styles (Optional[list[Style]]): The styles of the composite state.
        """
        super().__init__(id_, content, styles)
        self.sub_states: list[State] = sub_states if sub_states is not None else []
        self.transitions: list[BaseTransition] = (
            transitions if transitions is not None else []
        )
        self.direction: Optional[str] = (
            direction.value if isinstance(direction, Direction) else direction
        )

    def __str__(self) -> str:
        """Return the string representation of the state."""
        string: str = super().__str__()

        if len(self.sub_states):
            string += f"\nstate {self.id_} {{"
            if self.direction:
                string += f"\n\tdirection {self.direction}"
            for state in self.sub_states:
                string += f"\n\t{str(state)}"

            for transition in self.transitions:
                string += f"\n\t{str(transition)}"

            string += "\n}"

        return string


class Concurrent(Composite):
    """Concurrent class.

    This class represents a concurrent state in a state diagram.

    Attributes:
        groups (list[tuple[list[State],list[BaseTransition]]]): The groups of the concurrent state.
        styles (list[Style]): The styles of the concurrent state.
    """

    def __init__(
        self,
        id_: str,
        content: str = "",
        sub_groups: Optional[list[tuple[list[State], list[BaseTransition]]]] = None,
        styles: Optional[list[Style]] = None,
    ) -> None:
        """Initialize a new Concurrent.

        Args:
            id_ (str): The id of the state.
            content (str): The content of the state.
            sub_groups (Optional[list[tuple[list[State],list[BaseTransition]]]): The groups of the concurrent state.
            styles (Optional[list[Style]]): The styles of the concurrent state.
        """
        super().__init__(id_, content, styles=styles)
        self.groups: list[tuple[list[State], list[BaseTransition]]] = (
            sub_groups if sub_groups is not None else []
        )

    def __str__(self) -> str:
        """Return the string representation of the state."""
        string: str = super().__str__()
        if len(self.groups):
            string += f"\nstate {self.id_} {{"
            for states, transitions in self.groups:
                for state in states:
                    string += f"\n\t{str(state)}"
                for transition in transitions:
                    string += f"\n\t{str(transition)}"
                string += "\n\t--"
            string = string[:-4]
            string += "\n}"
        return string



from .task import Task


class Section:
    """Section class.

    This class represents a section in a user's journey.
    Each section contains a list of tasks.

    Attributes:
        name (str): The name of the section.
        tasks (list[Task]): The tasks in the section.
    """

    def __init__(self, name: str, tasks: list[Task]) -> None:
        """Initialize a new Section.

        Args:
            name (str): The name of the section.
            tasks (list[Task]): The tasks in the section.
        """
        self.name: str = name
        self.tasks: list[Task] = tasks

    def __str__(self) -> str:
        """Return a string representation of the section.

        Returns:
            str: A string representation of the section.
        """
        string: str = f"\tsection {self.name}\n"
        for task in self.tasks:
            string += f"{task}\n"
        return string



from enum import Enum
from typing import Union


class Risk(Enum):
    """Risk Enum.

    This enum represents the different levels of risk associated with a requirement in a requirement diagram.
    """

    LOW = "Low"
    MEDIUM = "Medium"
    HIGH = "High"


class VerifyMethod(Enum):
    """VerifyMethod Enum.

    This enum represents the different methods of verifying a requirement in a requirement diagram.
    """

    ANALYSIS = "Analysis"
    INSPECTION = "Inspection"
    TEST = "Test"
    DEMONSTRATION = "Demonstration"


class Type(Enum):
    """Return a string representation of the link.

    Returns:
        str: A string representation of the link.
    """

    REQUIREMENT = "requirement"
    FUNCTIONAL = "functionalRequirement"
    INTERFACE = "interfaceRequirement"
    PERFORMANCE = "performanceRequirement"
    PHYSICAL = "physicalRequirement"
    DESIGN_CONSTRAINT = "designConstraint"


class Requirement:
    """Requirement class.

    This class represents a requirement in a requirement diagram.

    Attributes:
        id_ (str): The ID of the requirement.
        name (str): The name of the requirement.
        text (str): The text of the requirement.
        type_ (Union[str, Type]): The type of the requirement.
        risk (Union[str, Risk]): The risk of the requirement.
        verifymethod (Union[str, VerifyMethod]): The verification method of the requirement.
    """

    def __init__(
        self,
        id_: str,
        name: str,
        text: str,
        type_: Union[str, Type],
        risk: Union[str, Risk],
        verifymethod: Union[str, VerifyMethod],
    ) -> None:
        """Initialize a new Requirement.

        Args:
            id_ (str): The ID of the requirement.
            name (str): The name of the requirement.
            text (str): The text of the requirement.
            type_ (Union[str, Type]): The type of the requirement.
            risk (Union[str, Risk]): The risk of the requirement.
            verifymethod (Union[str, VerifyMethod]): The verification method of the requirement.
        """
        self.id_: str = id_
        self.name: str = name
        self.text: str = text
        self.type_: str = type_ if isinstance(type_, str) else type_.value
        self.risk: str = risk if isinstance(risk, str) else risk.value
        self.verifymethod: str = (
            verifymethod if isinstance(verifymethod, str) else verifymethod.value
        )

    def __str__(self) -> str:
        return "".join(
            [
                f"{self.type_} {self.name} {{\n",
                f"\tid: {self.id_}\n",
                f"\ttext: {self.text}\n",
                f"\trisk: {self.risk}\n",
                f"\tverifymethod: {self.verifymethod}\n",
                "}\n",
            ]
        )



from .entity import Entity

LIST_CARDINALITIES: dict[str, list[str]] = {
    "zero-or-one": ["|o", "o|"],
    "exactly-one": ["||", "||"],
    "zero-or-more": ["}o", "o{"],
    "one-or-more": ["}|", "|{"],
}


class Link:
    """Link class.

    This class represents a link between entities in an ER diagram.

    Attributes:
        origin (Entity): The origin entity of the link.
        end (Entity): The end entity of the link.
        label (str): The label of the link.
        left_symbol (str): The symbol at the left end of the link.
        right_symbol (str): The symbol at the right end of the link.
        dotted (bool): Whether the link is dotted or not.
    """

    def __init__(
        self,
        origin: Entity,
        end: Entity,
        origin_cardinality: str,
        end_cardinality: str,
        label: str = "",
        dotted: bool = False,
    ) -> None:
        """Initialize a new Link.

        Args:
            origin (Entity): The origin entity of the link.
            end (Entity): The end entity of the link.
            label (str): The label of the link.
            origin_cardinality (str): The cardinality of the origin entity.
            end_cardinality (str): The cardinality of the end entity.
            dotted (bool): Whether the link is dotted or not.
        """
        self.origin: Entity = origin
        self.end: Entity = end
        self.label: str = label
        self.left_symbol: str = LIST_CARDINALITIES[origin_cardinality][0]
        self.right_symbol: str = LIST_CARDINALITIES[end_cardinality][1]
        self.dotted: bool = dotted

    def __str__(self) -> str:
        """Return a string representation of the link.

        The string representation includes the names of the origin and end entities,
        the symbols at both ends of the link, and the label of the link.

        Returns:
            str: A string representation of the link.
        """
        shape: str = ".." if self.dotted else "--"
        return f'{self.origin.name}{self.left_symbol}{shape}{self.right_symbol}{self.end.name} : "{self.label}"'



import unittest

from mermaid.configuration import Config, Themes


class TestThemes(unittest.TestCase):
    def test_themes(self):
        self.assertEqual("default", Themes.DEFAULT.value)
        self.assertEqual("forest", Themes.FOREST.value)
        self.assertEqual("dark", Themes.DARK.value)
        self.assertEqual("neutral", Themes.NEUTRAL.value)
        self.assertEqual("base", Themes.BASE.value)


class TestConfig(unittest.TestCase):
    def test_default_config(self):
        config = Config()
        expect_str = """%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t}
\t}
}%%
"""
        self.assertEqual(expect_str, str(config))

    def test_config_with_primary_color(self):
        config = Config(primary_color="red")
        expect_str = """%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t\t"primaryColor": "red"
\t\t}
\t}
}%%
"""
        self.assertEqual(expect_str, str(config))

    def test_config_with_primary_text_color(self):
        config = Config(primary_text_color="red")
        expect_str = """%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t\t"primaryTextColor": "red"
\t\t}
\t}
}%%
"""
        self.assertEqual(expect_str, str(config))

    def test_config_with_all_colors(self):
        config = Config(
            primary_color="red",
            primary_text_color="red",
            primary_border_color="red",
            line_color="red",
            secondary_color="red",
            tertiary_color="red",
        )
        expect_str = """%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t\t"primaryColor": "red",
\t\t\t"primaryTextColor": "red",
\t\t\t"primaryBorderColor": "red",
\t\t\t"lineColor": "red",
\t\t\t"secondaryColor": "red",
\t\t\t"tertiaryColor": "red"
\t\t}
\t}
}%%
"""
        self.assertEqual(expect_str, str(config))

    def test_config_with_non_default_theme(self):
        config = Config(theme=Themes.FOREST)
        expect_str = """%%{
\tinit: {
\t\t"theme": "forest",
\t\t"themeVariables": {
\t\t}
\t}
}%%
"""
        self.assertEqual(expect_str, str(config))



"""Sequence diagram elements."""

from enum import Enum
from typing import Union

from mermaid.utils import text_to_snake_case


class Actor:
    """Actor class for mermaid sequence diagram.

    Args:
        name (str): Name of the actor.
    """

    def __init__(self, name: str):
        """Initialize actor.

        Args:
            name (str): Name of the actor.
        """
        self.name: str = name
        self.id_: str = name

    def __str__(self):
        """Return actor string.

        Returns:
            str: Actor string.
        """
        return f"\tactor {self.name}\n"


class Participant:
    """Participant class for mermaid sequence diagram.

    Args:
        name (str): Name of the participant.
    """

    def __init__(self, name: str):
        """Initialize participant.

        Args:
            name (str): Name of the participant.
        """
        self.name: str = name
        self.id_: str = text_to_snake_case(self.name)

    def __str__(self):
        """Return participant string.

        Returns:
            str: Participant string.
        """
        return f"\tparticipant {self.id_} as {self.name}\n"


class Box:
    """Box class for mermaid sequence diagram.

    Args:
        elements (List[Union[Actor, Participant]]): List of Actor or Participant objects.
        name (str): Name of the box.
    """

    def __init__(self, name: str, elements: list[Union[Actor, Participant]]):
        self.elements = elements
        self.name = name

    def __str__(self):
        """Return box string.

        Returns:
            str: Box string.
        """
        box_str = f"\tbox {self.name}\n"
        for element in self.elements:
            box_str += str(element)
        box_str += "\tend\n"
        return box_str


class NotePosition(Enum):
    """Note position enum for mermaid sequence diagram.

    Args:
        Enum (str): Note position.
    """

    LEFT_OF = "left of"
    RIGHT_OF = "right of"
    OVER = "over"


class Note:
    """Note class for mermaid sequence diagram.

    Args:
        note (str): Note to be displayed.
        actor (Union[Actor, Participant], optional): Actor or Participant to attach the note to. Defaults to None.
    """

    def __init__(
        self,
        note: str,
        element: Union[list[Union[Actor, Participant]], Union[Actor, Participant]],
        position: Union[NotePosition, str] = NotePosition.OVER,
    ) -> None:
        self.note = note
        self.element = (
            element[0] if isinstance(element, list) and len(element) == 1 else element
        )
        self.position = position if isinstance(position, str) else position.value
        if self.position in [
            NotePosition.RIGHT_OF.value,
            NotePosition.LEFT_OF.value,
        ] and isinstance(self.element, list):
            raise ValueError(
                f"Note position {self.position} cannot be used with multiple elements"
            )

    def __str__(self):
        """Return note string.

        Returns:
            str: Note string.
        """
        if isinstance(self.element, list):
            note_str = f"\tNote {self.position} "
            for element in self.element:
                note_str += f"{element.id_},"
            note_str = note_str[:-1]
            note_str += f": {self.note}\n"
            return note_str
        else:
            return f"\tNote {self.position} {self.element.id_}: {self.note}\n"


class Rect:
    """Rect class for mermaid sequence diagram.

    Args:
        elements (List[Union[Link, Logic, Rect]]): List of Link, Logic or Rect objects.
        color (tuple[int, ...]): RGB color tuple.
    """

    def __init__(self, elements: list["Rect"], color: tuple[int, ...]) -> None:
        # FIXME: Add type hints for Link and Logic to include links and Logics
        # and avoid the circular import
        """Initialize rect.

        Args:
            elements (List[Union[Link, Logic, Rect]]): List of Link, Logic or Rect objects.
            color (tuple[int, ...]): RGB color tuple.
        """
        self.elements = elements
        self.color = color
        if len(color) != 3 or not all((x >= 0 and x <= 255) for x in color):
            raise ValueError("color must be a tuple of 3 integers between 0 and 255")

    def __str__(self) -> str:
        """Return rect string.

        Returns:
            str: Rect string.
        """
        rect_str = f"\trect rgb({self.color[0]},{self.color[1]},{self.color[2]})\n"
        for element in self.elements:
            rect_str += str(element)
        rect_str += "\tend\n"
        return rect_str



"""FlowChart module.

This module provides the FlowChart class for creating and manipulating flowcharts.
It also provides utility classes for representing nodes and links in a flowchart.

Classes:
    FlowChart: Represents a flowchart.
    Node: Represents a node in a flowchart.
    Link: Represents a link between nodes in a flowchart.
"""

from typing import Optional, Union

from mermaid import Direction
from mermaid.configuration import Config
from mermaid.graph import Graph
from mermaid.style import Style

from .link import Link, LinkHead, LinkShape
from .node import Node


class FlowChart(Graph):
    """FlowChart class.

    This class represents a flowchart.

    Attributes:
        orientation (str): The orientation of the flowchart.
        nodes (list[Node]): The nodes in the flowchart.
        links (list[Link]): The links between nodes in the flowchart.
        orientation (str): The orientation of the flowchart.
        config (Config): The configuration for the flowchart.
    """

    def __init__(
        self,
        title: str,
        nodes: Optional[list[Node]] = None,
        links: Optional[list[Link]] = None,
        orientation: Union[str, Direction] = "TB",
        config: Optional[Config] = None,
    ) -> None:
        """Initialize a new FlowChart.

        Args:
            title (str): The title of the flowchart.
            nodes (Optional[list[Node]]): The nodes in the flowchart.
            links (Optional[list[Link]]): The links between nodes in the flowchart.
            orientation (str): The orientation of the flowchart.
            config (Optional[Config]): The configuration for the flowchart.
        """
        super().__init__(title, "", config)
        self.orientation: str = (
            orientation if isinstance(orientation, str) else orientation.value
        )
        self.nodes: list[Node] = nodes if nodes is not None else []
        self.links: list[Link] = links if links is not None else []
        self.styles: set[Style] = set()
        for node in self.nodes:
            self.styles.update(node.styles)
        self._build_script()

    def _build_script(self) -> None:
        super()._build_script()
        script: str = f"\nflowchart {self.orientation}"
        for style in self.styles:
            script += f"\n\t{style}"

        for node in self.nodes:
            script += f"\n\t{node}"
        for link in self.links:
            script += f"\n\t{link}"
        self.script += script + "\n"


__all__ = [
    "FlowChart",
    "Node",
    "Link",
    "LinkHead",
    "LinkShape",
]



from dataclasses import dataclass


@dataclass
class Actor:
    """Actor class.

    This class represents an actor in a movie or a play.

    Attributes:
        name (str): The name of the actor.
    """

    name: str



from typing import Optional


class Element:
    """UserJourney module.

    This module provides the UserJourney class for creating and manipulating user journeys.
    It also provides the Section and Task classes for representing sections and tasks in a user journey.

    Classes:
        UserJourney: Represents a user's journey through a website or application.
        Section: Represents a section in a user's journey.
        Task: Represents a task in a section of a user's journey.
    """

    def __init__(self, name: str, type_: str, docRef: Optional[str] = None) -> None:
        """Initialize a new Element.

        Args:
            name (str): The name of the element.
            type_ (str): The type of the element.
            docRef (Optional[str]): The documentation reference for the element.
        """
        self.name: str = name
        self.type_: str = type_
        self.docRef: Optional[str] = docRef

    def __str__(self) -> str:
        """Return a string representation of the element.

        Returns:
            str: A string representation of the element.
        """
        string: str = f"element {self.name} {{\n"
        string += f'\ttype: "{self.type_}"\n'
        string += f"\tdocRef: {self.docRef}\n" if self.docRef else ""
        string += "}\n"
        return string



import unittest
from unittest import mock

from mermaid.configuration import Config
from mermaid.reqdiagram import (
    Element,
    Link,
    Requirement,
    RequirementDiagram,
    Risk,
    Type,
    VerifyMethod,
)


class TestRequirement(unittest.TestCase):
    def test_create_requirement_with_str(self):
        requirement = Requirement(
            "1", "test_req", "the test text.", "requirement", "high", "Test"
        )
        expect_str = """requirement test_req {
\tid: 1
\ttext: the test text.
\trisk: high
\tverifymethod: Test
}
"""
        self.assertEqual(expect_str, str(requirement))

    def test_create_requirement_with_enum(self):
        requirement = Requirement(
            "1",
            "test_req",
            "the test text.",
            Type.INTERFACE,
            Risk.LOW,
            VerifyMethod.ANALYSIS,
        )
        expect_str = """interfaceRequirement test_req {
\tid: 1
\ttext: the test text.
\trisk: Low
\tverifymethod: Analysis
}
"""
        self.assertEqual(expect_str, str(requirement))


class TestElement(unittest.TestCase):
    def test_element_with_docref(self):
        element = Element("test_entity", "simulation", "/test/test_....py")
        expect_str = """element test_entity {
\ttype: "simulation"
\tdocRef: /test/test_....py
}
"""
        self.assertEqual(expect_str, str(element))

    def test_element_without_docref(self):
        element = Element("test_entity", "simulation")
        expect_str = """element test_entity {
\ttype: "simulation"
}
"""
        self.assertEqual(expect_str, str(element))


class TestLink(unittest.TestCase):
    def test_string_link_with_element_requirement(self):
        element = mock.Mock()
        element.name = "test_entity"
        requirement = mock.Mock()
        requirement.name = "test_req"
        link = Link(element, requirement, "traces")
        expect_str = "test_entity - traces -> test_req"

        self.assertEqual(expect_str, str(link))

    def test_string_link_with_requirements(self):
        requirement_1 = mock.Mock()
        requirement_1.name = "test_req"
        requirement_2 = mock.Mock()
        requirement_2.name = "test_req2"
        link = Link(requirement_1, requirement_2, "contains")
        expect_str = "test_req - contains -> test_req2"

        self.assertEqual(expect_str, str(link))


class TestRequirementDiagram(unittest.TestCase):
    def test_script_diagram(self):
        elements = [
            Element("test_entity_1", "simulation"),
            Element("test_entity_2", "simulation"),
        ]
        requirements = [
            Requirement(
                "1.1", "test_req_1", "the test text.", "requirement", "high", "Test"
            ),
            Requirement(
                "1.2", "test_req_2", "the test text.", "requirement", "high", "Test"
            ),
        ]
        links = [
            Link(elements[0], requirements[0], "traces"),
            Link(elements[1], requirements[1], "traces"),
            Link(requirements[0], requirements[1], "contains"),
        ]
        diagram = RequirementDiagram(
            "simple requirement", elements, requirements, links
        )
        expect_str = f"""---
title: simple requirement
---
requirementDiagram
{elements[0]}
{elements[1]}
{requirements[0]}
{requirements[1]}
{links[0]}
{links[1]}
{links[2]}
"""
        self.assertEqual(expect_str, diagram.script)

    def test_script_diagram_with_config(self):
        elements = [
            Element("test_entity_1", "simulation"),
            Element("test_entity_2", "simulation"),
        ]
        requirements = [
            Requirement(
                "1.1", "test_req_1", "the test text.", "requirement", "high", "Test"
            ),
            Requirement(
                "1.2", "test_req_2", "the test text.", "requirement", "high", "Test"
            ),
        ]
        links = [
            Link(elements[0], requirements[0], "traces"),
            Link(elements[1], requirements[1], "traces"),
            Link(requirements[0], requirements[1], "contains"),
        ]
        config = Config(primary_color="red")
        diagram = RequirementDiagram(
            "simple requirement", elements, requirements, links, config
        )
        sub_string = """---
title: simple requirement
---
%%{
\tinit: {
\t\t"theme": "default",
\t\t"themeVariables": {
\t\t\t"primaryColor": "red"
\t\t}
\t}
}%%
"""
        expect_str = f"""{sub_string}
requirementDiagram
{elements[0]}
{elements[1]}
{requirements[0]}
{requirements[1]}
{links[0]}
{links[1]}
{links[2]}
"""
        self.assertEqual(expect_str, diagram.script)



"""
This module contains the PieChart class which extends the Graph class
from mermaid.graph module.

The PieChart class represents a PieChart diagram and provides methods
to build the Mermaid diagram script for the pie chart.

Classes:
    PieChart: Represents a PieChart diagram.
"""

from typing import Optional

from mermaid.configuration import Config
from mermaid.graph import Graph


class PieChart(Graph):
    """
    This class represents a PieChart diagram.

    Attributes:
        data (dict[str, float]): The data for the pie chart.
        show_data (bool): Whether to show data on the pie chart.
        config (Config): The configuration for the pie chart.
    """

    def __init__(
        self,
        title: str,
        data: dict[str, float],
        show_data: bool = False,
        config: Optional[Config] = None,
    ) -> None:
        """
        The constructor for the PieChart class.

        Parameters:
            title (str): The title of the pie chart.
            data (dict[str, float]): The data for the pie chart.
            show_data (bool): Whether to show data on the pie chart. Defaults to False.
            config (Optional[Config]): The configuration for the pie chart. Defaults to None.
        """
        super().__init__(title, "", config)
        self.data: dict[str, float] = data
        self.show_data: bool = show_data
        self._build_script()

    def _build_script(self) -> None:
        """
        Build the Mermaid diagram script for the pie chart.
        """
        super()._build_script()
        showData: str = " showData" if self.show_data else ""
        script: str = f"\npie{showData}"

        for key, value in self.data.items():
            script += f'\n\t"{key}" : {value}'

        self.script += script + "\n"



import unittest

from mermaid.configuration import Config, Themes
from mermaid.erdiagram import Entity, ERDiagram, Link  # noqa: I101


class TestEnity(unittest.TestCase):
    def test_str_entity_without_addition_info(self):
        attributes = {"id": "int", "salary": "float", "name": "string"}
        entity = Entity("Employee", attributes)
        expect_str = """Employee{
\tint id
\tfloat salary
\tstring name
}"""
        self.assertEqual(expect_str, str(entity))

    def test_str_entity_with_addition_info(self):
        attributes = {
            "id": ["int", "PK"],
            "salary": "float",
            "id_cos": ["int", "FK", "comment"],
            "name": ["string", "comment"],
        }
        entity = Entity("Employee", attributes)
        expect_str = """Employee{
\tint id PK
\tfloat salary
\tint id_cos FK "comment"
\tstring name "comment"
}"""
        self.assertEqual(expect_str, str(entity))

    def test_update_attributes(self):
        attributes = {"id": "int", "name": "string"}
        entity = Entity("Employee", attributes)
        entity.update_attributes({"age": "float", "id": ["int", "PK"]})
        expect_attributes = {"id": ["int", "PK"], "name": "string", "age": "float"}

        self.assertEqual(expect_attributes, entity.attributes)

    def test_add_attribute(self):
        attributes = {"id": "int"}
        entity = Entity("Employee", attributes)
        entity.add_attribute("age", "float")
        entity.add_attribute("id_cos", "int", constraint="FK")
        entity.add_attribute("phone", "string", constraint="UK", comment="phone number")
        expect_attributes = {
            "id": "int",
            "age": ["float"],
            "id_cos": ["int", "FK"],
            "phone": ["string", "UK", "phone number"],
        }

        self.assertEqual(expect_attributes, entity.attributes)


class TestLink(unittest.TestCase):
    def setUp(self) -> None:
        self.enity_1 = Entity("User")
        self.enity_2 = Entity("Tag")

    def test_link_without_label(self):
        link = Link(self.enity_1, self.enity_2, "exactly-one", "zero-or-more")
        expect_string = 'User||--o{Tag : ""'
        self.assertEqual(expect_string, str(link))

    def test_link_with_label(self):
        link = Link(
            self.enity_1, self.enity_2, "exactly-one", "zero-or-more", label="has"
        )
        expect_string = 'User||--o{Tag : "has"'
        self.assertEqual(expect_string, str(link))

    def test_dotted_link(self):
        link = Link(
            self.enity_1, self.enity_2, "exactly-one", "zero-or-more", dotted=True
        )
        expect_string = 'User||..o{Tag : ""'
        self.assertEqual(expect_string, str(link))


class TestERDiagram(unittest.TestCase):
    def test_script_erdiagram(self):
        entity_1 = Entity("User", {"id": ["int", "PK"], "name": ["string", "the name"]})
        entity_2 = Entity("Tag", {"id": ["int", "PK"], "name": "string"})
        entities = [entity_1, entity_2]
        links = [Link(entity_1, entity_2, "exactly-one", "zero-or-more", dotted=True)]
        diagram = ERDiagram("e-commerce website", entities, links)
        expect_str = f"""---
title: e-commerce website
---
erDiagram
\t{entities[0]}
\t{entities[1]}
\t{links[0]}
"""
        self.assertEqual(expect_str, diagram.script)

    def test_script_erdiagram_with_config(self):
        entity_1 = Entity("User", {"id": ["int", "PK"], "name": ["string", "the name"]})
        entity_2 = Entity("Tag", {"id": ["int", "PK"], "name": "string"})
        entities = [entity_1, entity_2]
        links = [Link(entity_1, entity_2, "exactly-one", "zero-or-more", dotted=True)]
        config = Config(theme=Themes.DARK, primary_color="red")
        diagram = ERDiagram("e-commerce website", entities, links, config)
        sub_string = """---
title: e-commerce website
---
%%{
\tinit: {
\t\t"theme": "dark",
\t\t"themeVariables": {
\t\t\t"primaryColor": "red"
\t\t}
\t}
}%%
"""
        expect_string = f"""{sub_string}
erDiagram
\t{entities[0]}
\t{entities[1]}
\t{links[0]}
"""
        self.assertEqual(expect_string, diagram.script)



from typing import Union

from mermaid.sequence.link import Link


class Logic:
    """Logic class for mermaid sequence diagram."""

    pass


class Loop(Logic):
    """Loop class for mermaid sequence diagram.

    Args:
        condition (str): Condition of the loop.
        link (List[Link]): List of Link objects.
    """

    def __init__(self, condition: str, link: list[Link]):
        """Initialize loop.

        Args:
            condition (str): Condition of the loop.
            link (List[Link]): List of Link objects.
        """
        self.condition = condition
        self.link = link

    def __str__(self) -> str:
        """Return loop string.

        Returns:
            str: Loop string.
        """
        return (
            f"\tloop {self.condition}\n"
            + "".join([str(link) for link in self.link])
            + "\tend\n"
        )


class Alt(Logic):
    """Alt class for mermaid sequence diagram.

    Args:
        condition_links (dict[str, List[Link]]): Dictionary of condition
        and list of Link objects.
    """

    def __init__(self, condition_links: dict[str, list[Link]]):
        """Initialize alt.

        Args:
            condition_links (dict[str, List[Link]]): Dictionary of condition
            and list of Link objects.
        """
        self.condition_links = condition_links

    def __str__(self) -> str:
        """Return alt string.

        Returns:
            str: Alt string.
        """
        alt_str = ""
        n = 1
        for condition, links in self.condition_links.items():
            if n == 1:
                alt_str += f"\talt {condition}\n" + "".join(
                    [str(link) for link in links]
                )
                n += 1
            else:
                alt_str += f"\telse {condition}\n" + "".join(
                    [str(link) for link in links]
                )

        alt_str += "\tend\n"
        return alt_str


class Optional(Logic):
    """Optional class for mermaid sequence diagram.

    Args:
        condition (str): Condition of the loop.
        statements (List[Link]): List of Link objects.
    """

    def __init__(self, condition: str, statements: list[Link]) -> None:
        """Initialize loop.

        Args:
            condition (str): Condition of the loop.
            statements (List[Link]): List of Link objects.
        """
        self.condition = condition
        self.statements = statements

    def __str__(self) -> str:
        """Return loop string.

        Returns:
            str: Loop string.
        """
        return (
            f"\topt {self.condition}\n"
            + "".join([str(statement) for statement in self.statements])
            + "\tend\n"
        )


class Parallel:
    """Parallel class for mermaid sequence diagram.

    Args:
        condition_elements (dict[str, list[Union[Link, Logic]]]): Dictionary of
        condition and Link or Logic objects.
    """

    def __init__(self, condition_elements: dict[str, list[Union[Link, Logic]]]) -> None:
        """Initialize parallel.

        Args:
            condition_elements (dict[str, list[Union[Link, Logic]]]): Dictionary of
            condition and Link or Logic objects.
        """
        self.condition_elements = condition_elements

    def __str__(self) -> str:
        """Return parallel string.

        Returns:
            str: Parallel string.
        """
        parallel_str = ""
        n = 1
        for condition, elements in self.condition_elements.items():
            if n == 1:
                parallel_str += f"\tpar {condition}\n" + "".join(
                    [str(element) for element in elements]
                )
                n += 1
            else:
                parallel_str += f"\tand {condition}\n" + "".join(
                    [str(element) for element in elements]
                )
        parallel_str += "\tend\n"
        return parallel_str


class Critical(Logic):
    """Critical class for mermaid sequence diagram.

    Args:
        condition (str): Condition of the loop.
        link (List[Link]): List of Link objects.
    """

    def __init__(
        self,
        condition: str,
        statements: list[Union[Link, Logic]],
        optional_statements: dict[str, list[Union[Link, Logic]]],
    ) -> None:
        """Initialize loop.

        Args:
            condition (str): Condition of the loop.
            link (List[Link]): List of Link objects.
        """
        self.condition = condition
        self.statements = statements
        self.optional_statements = optional_statements

    def __str__(self) -> str:
        """Return loop string.

        Returns:
            str: Loop string.
        """
        critical_str = f"\tcritical {self.condition}\n" + "".join(
            [str(statement) for statement in self.statements]
        )
        for condition, statements in self.optional_statements.items():
            critical_str += f"\toption {condition}\n" + "".join(
                [str(statement) for statement in statements]
            )
        critical_str += "\tend\n"
        return critical_str


class Break(Logic):
    """Break class for mermaid sequence diagram.

    Args:
        condition (str): Condition of the loop.
        link (List[Link]): List of Link objects.
    """

    def __init__(self, condition: str, statements: list[Union[Link, Logic]]) -> None:
        """Initialize loop.

        Args:
            condition (str): Condition of the loop.
            link (List[Link]): List of Link objects.
        """
        self.condition = condition
        self.statements = statements

    def __str__(self) -> str:
        """Return loop string.

        Returns:
            str: Loop string.
        """
        break_str = f"\tbreak {self.condition}\n" + "".join(
            [str(statement) for statement in self.statements]
        )
        break_str += "\tend\n"
        return break_str



from dataclasses import dataclass
from typing import Optional, Union

from mermaid import Direction, text_to_snake_case
from mermaid.style import Style


@dataclass
class NodeShape:
    """NodeShape class.

    This class represents the shape of a node in a flowchart.

    Attributes:
        start (str): The start character of the node shape.
        end (str): The end character of the node shape.
    """

    start: str
    end: str


NODE_SHAPES: dict[str, NodeShape] = {
    "normal": NodeShape("[", "]"),
    "round-edge": NodeShape("(", ")"),
    "stadium-shape": NodeShape("([", "])"),
    "subroutine-shape": NodeShape("[[", "]]"),
    "cylindrical": NodeShape("[(", ")]"),
    "circle": NodeShape("((", "))"),
    "label-shape": NodeShape(">", "]"),
    "rhombus": NodeShape("{", "}"),
    "hexagon": NodeShape("{{", "}}"),
    "parallelogram": NodeShape("[/", "/]"),
    "parallelogram-alt": NodeShape("[\\", "\\]"),
    "trapezoid": NodeShape("[/", "\\]"),
    "trapezoid-alt": NodeShape("[\\", "/]"),
    "double-circle": NodeShape("(((", ")))"),
}

HREF_TYPES: dict[str, str] = {
    "blank": "_blank",
    "self": "_self",
    "parent": "_parent",
    "top": "_top",
}


class Node:
    """Node class.

    This class represents a node in a flowchart.

    Attributes:
        id_ (str): The ID of the node.
        content (str): The content of the node.
        shape (str): The shape of the node.
        sub_nodes (list[Node]): The sub-nodes of the node.
        href (str): The hyperlink reference of the node.
        href_type (str): The type of the hyperlink reference of the node.
        styles (list[Style]): The styles of the node.
    """

    def __init__(
        self,
        id_: str,
        content: str = "",
        shape: str = "normal",
        sub_nodes: Optional[list["Node"]] = None,
        href: Optional[str] = None,
        href_type: str = "blank",
        styles: Optional[list[Style]] = None,
        direction: Union[str, Direction] = "LR",
    ) -> None:
        """Initialize a new Node.

        Args:
            id_ (str): The ID of the node.
            content (str): The content of the node.
            shape (str): The shape of the node.
            sub_nodes (Optional[list[Node]]): The sub-nodes of the node.
            href (Optional[str]): The hyperlink reference of the node.
            href_type (str): The type of the hyperlink reference of the node.
            styles (Optional[list[Style]]): The styles of the node.
            direction (Optional[Union[str,Direction]]): The direction of the node in case of a subgraph.
        """
        self.id_: str = text_to_snake_case(id_)
        self.content: str = content if content else id_
        self.shape: NodeShape = NODE_SHAPES[shape]
        self.href: str = href if href is not None else "#"
        self.href_type: str = HREF_TYPES[href_type]
        self.sub_nodes: list["Node"] = sub_nodes if sub_nodes is not None else []
        self.styles: list[Style] = styles if styles is not None else []
        self.direction: str = (
            direction if isinstance(direction, str) else direction.value
        )

    def __str__(self) -> str:
        """Return a string representation of the node.

        If the node has sub-nodes, it returns a string representation of a subgraph.
        Otherwise, it returns a string representation of a single node.

        Returns:
            str: A string representation of the node.
        """
        string: str = ""
        if len(self.sub_nodes):
            string = "\n".join(
                [
                    f'subgraph {self.id_} ["{self.content}"]',
                    f"\tdirection {self.direction}",
                    "\n".join([str(f"\t{node}") for node in self.sub_nodes]),
                    "end",
                ]
            )
        else:
            string = "".join(
                [self.id_, self.shape.start, f'"{self.content}"', self.shape.end]
            )
            if self.href != "#":
                string = "".join(
                    [string, "\n", f'click {self.id_} "{self.href}" {self.href_type}']
                )

        for style in self.styles:
            string += f"\n{self.id_}:::{style.name}"
        return string



from enum import Enum
from typing import Optional


class Themes(Enum):
    """Enum class for themes

    args:
        Enum: Enum class

    methods:
        value: returns the value of the Enum class
    """

    DEFAULT = "default"
    FOREST = "forest"
    DARK = "dark"
    NEUTRAL = "neutral"
    BASE = "base"


class Config:
    """Config class for mermaid diagrams

    args:
        theme: Themes = Themes.DEFAULT
        primary_color: Optional[str] = None
        primary_text_color: Optional[str] = None
        primary_border_color: Optional[str] = None
        line_color: Optional[str] = None
        secondary_color: Optional[str] = None
        tertiary_color: Optional[str] = None

    methods:
        __init__: initializes the class
        __str__: returns the string representation of the class
    """

    def __init__(
        self,
        theme: Themes = Themes.DEFAULT,
        primary_color: Optional[str] = None,
        primary_text_color: Optional[str] = None,
        primary_border_color: Optional[str] = None,
        line_color: Optional[str] = None,
        secondary_color: Optional[str] = None,
        tertiary_color: Optional[str] = None,
    ) -> None:
        """Initializes the class

        args:
            theme: Themes = Themes.DEFAULT
            primary_color: Optional[str] = None
            primary_text_color: Optional[str] = None
            primary_border_color: Optional[str] = None
            line_color: Optional[str] = None
            secondary_color: Optional[str] = None
            tertiary_color: Optional[str] = None
        """
        self.theme: Themes = theme
        self.primary_color: Optional[str] = primary_color
        self.primary_text_color: Optional[str] = primary_text_color
        self.primary_border_color: Optional[str] = primary_border_color
        self.line_color: Optional[str] = line_color
        self.secondary_color: Optional[str] = secondary_color
        self.tertiary_color: Optional[str] = tertiary_color

    def __str__(self) -> str:
        """Returns the string representation of the class

        returns:
            str: string representation of the class
        """
        string: str = "%%{\n\tinit: {\n"

        string += f'\t\t"theme": "{self.theme.value}",\n\t\t"themeVariables": ' + "{\n"
        string += "".join(
            [
                f'\t\t\t"{style}": "{value}",\n'
                for style, value in [
                    ("primaryColor", self.primary_color),
                    ("primaryTextColor", self.primary_text_color),
                    ("primaryBorderColor", self.primary_border_color),
                    ("lineColor", self.line_color),
                    ("secondaryColor", self.secondary_color),
                    ("tertiaryColor", self.tertiary_color),
                ]
                if value
            ]
        )
        if string.endswith(",\n"):
            string = string[:-2] + "\n"
        string += "\t\t}\n"
        string += "\t}\n}%%\n"
        return string



"""
StateDiagram class.

This module contains the StateDiagram class.
"""

from typing import Optional, Union

from mermaid import Direction
from mermaid.configuration import Config
from mermaid.graph import Graph
from mermaid.style import Style

from .base import BaseTransition
from .state import Composite, Concurrent, End, Start, State
from .transition import Choice, Fork, Join, Transition


class StateDiagram(Graph):
    """stateDiagram class.

    Args:
        title (str): Title of the stateDiagram.
        states (list[State]): List of states.
        transitions (list[BaseTransition]): List of transitions.
        version (str, optional): Version of the stateDiagram. Defaults to 'v2'.
        direction (Optional[Union[str,Direction]], optional): Direction of the stateDiagram. Defaults to None.
        config (Optional[Config], optional): Configuration for the stateDiagram. Defaults to None.
    """

    def __init__(
        self,
        title: str,
        states: Optional[list[State]] = None,
        transitions: Optional[list[BaseTransition]] = None,
        version: str = "v2",
        direction: Optional[Union[str, Direction]] = None,
        config: Optional[Config] = None,
    ) -> None:
        """stateDiagram class.

        Args:
            title (str): Title of the stateDiagram.
            states (list[State]): List of states.
            transitions (list[BaseTransition]): List of transitions.
            version (str, optional): Version of the stateDiagram. Defaults to 'v2'.
            direction (Optional[Union[str,Direction]], optional): Direction of the stateDiagram. Defaults to None.
            config (Optional[Config], optional): Configuration for the stateDiagram. Defaults to None.
        """
        super().__init__(title, "", config)
        self.states: list[State] = states if states is not None else []
        self.transitions: list[BaseTransition] = (
            transitions if transitions is not None else []
        )
        self.version: str = version
        self.styles: set[Style] = set()
        for state in self.states:
            self.styles.update(state.styles)

        self.direction: Optional[str] = None

        if direction:
            self.direction = (
                direction if isinstance(direction, str) else direction.value
            )

        self._build_script()

    def _build_script(self) -> None:
        super()._build_script()
        str_version: str = f"-{self.version}" if self.version != "v1" else ""
        script: str = f"\nstateDiagram{str_version}"
        if self.direction:
            script += f"\n\tdirection {self.direction}"
        for style in self.styles:
            script += f"\n\t{style}"
        for state in self.states:
            script += f"\n\t{state}"
        for transition in self.transitions:
            script += f"\n\t{transition}"

        self.script += script + "\n"


__all__ = [
    "StateDiagram",
    "State",
    "Composite",
    "Concurrent",
    "Start",
    "End",
    "Transition",
    "Choice",
    "Fork",
    "Join",
]



import os
import re
from pathlib import Path
from typing import Optional

from .graph import Graph


def load(path: Path) -> Graph:
    """
    Load a Mermaid diagram script from a file and return a Graph object.

    Parameters:
        path (Path): The path of the file to load from.

    Returns:
        Graph: The Graph object containing the Mermaid diagram script and the name of the file.
    """
    script: Optional[str] = None
    name: Optional[str] = None

    if os.path.exists(path):
        with open(path, "r") as file:
            script = file.read()
    else:
        raise FileNotFoundError(f"could not find a file in path::{path}")

    file_name: str = os.path.basename(path)
    name, _ = os.path.splitext(file_name)

    return Graph(name, script)


def text_to_snake_case(text: str) -> str:
    """
    Convert a string to snake_case.

    Parameters:
        text (str): The string to convert.

    Returns:
        str: The converted string.
    """
    # Remove non-alphanumeric characters except underscores, dots and dashes
    #  and replace everything else with underscores
    out: str = re.sub(r"[^a-zA-Z0-9_\.-]", "_", text)

    # Convert the text to lowercase
    out = out.lower()

    return out



"""ERDiagram module.

This module provides the ERDiagram class for creating and manipulating ER diagrams.
It also provides utility classes for representing entities and links in an ER diagram.

Classes:
    ERDiagram: Represents an ER diagram.
    Entity: Represents an entity in an ER diagram.
    Link: Represents a link between entities in an ER diagram.
"""

from typing import Optional

from mermaid.configuration import Config
from mermaid.erdiagram.entity import Entity
from mermaid.erdiagram.link import LIST_CARDINALITIES, Link
from mermaid.graph import Graph


class ERDiagram(Graph):
    """ERDiagram class.

    This class represents an ER diagram.

    Attributes:
        title (str): The title of the ER diagram.
        entities (list[Entity]): The entities of the ER diagram.
        links (list[Link]): The links of the ER diagram.
        config (Optional[Config]): The configuration for the ER diagram.
    """

    def __init__(
        self,
        title: str,
        entities: Optional[list[Entity]] = None,
        links: Optional[list[Link]] = None,
        config: Optional[Config] = None,
    ) -> None:
        """Initialize a new ERDiagram.

        Args:
            title (str): The title of the ER diagram.
            entities (Optional[list[Entity]]): The entities of the ER diagram.
            links (Optional[list[Link]]): The links of the ER diagram.
            config (Optional[Config]): The configuration for the ER diagram.
        """
        super().__init__(title, "", config)
        self.entities: list[Entity] = entities if entities is not None else []
        self.links: list[Link] = links if links is not None else []
        self._build_script()

    def _build_script(self) -> None:
        """Build the script of the ER diagram."""
        super()._build_script()
        script: str = "\nerDiagram"
        for entity in self.entities:
            script += f"\n\t{entity}"
        for link in self.links:
            script += f"\n\t{link}"
        self.script += script + "\n"


__all__ = ["ERDiagram", "Entity", "Link", "LIST_CARDINALITIES"]



from dataclasses import dataclass


@dataclass
class Icon:
    name: str
    type_: str
    version: str = "v1"

    def __str__(self) -> str:
        if self.version == "v1":
            return f"{self.type_} {self.name}"
        elif self.version == "v2":
            return f"{self.type_}:{self.name}"
        return f" {self.type_}:{self.name} "



import unittest

from mermaid.icon import Icon


class TestIcon(unittest.TestCase):
    def setUp(self) -> None:
        self.icon_v1 = Icon("icon", "fa")
        self.icon_v2 = Icon("icon", "fa", version="v2")
        self.icon_v3 = Icon("icon", "fa", version="v3")

    def test_str(self):
        self.assertEqual(str(self.icon_v1), "fa icon")
        self.assertEqual(str(self.icon_v2), "fa:icon")
        self.assertEqual(str(self.icon_v3), " fa:icon ")



"""ReqDiagram module.

This module provides the ReqDiagram class for creating and manipulating requirement diagrams.
It also provides utility classes for representing elements, links, and requirements in a requirement diagram.

Classes:
    ReqDiagram: Represents a requirement diagram.
    Element: Represents an element in a requirement diagram.
    Link: Represents a link between elements or requirements in a requirement diagram.
    Requirement: Represents a requirement in a requirement diagram.
"""

from typing import Optional

from mermaid.configuration import Config
from mermaid.graph import Graph

from .element import Element
from .link import Link
from .requirement import Requirement, Risk, Type, VerifyMethod


class RequirementDiagram(Graph):
    """RequirementDiagram class.

    This class represents a requirement diagram, which is a type of graph.
    It contains elements, requirements, and links between them.

    Attributes:
        elements (list[Element]): The elements in the diagram.
        requirements (list[Requirement]): The requirements in the diagram.
        links (list[Link]): The links between elements and requirements in the diagram.
        config (Config): The configuration for the requirement diagram.
    """

    def __init__(
        self,
        title: str,
        elements: list[Element],
        requirements: list[Requirement],
        links: list[Link],
        config: Optional[Config] = None,
    ) -> None:
        """Initialize a new RequirementDiagram.

        Args:
            title (str): The title of the diagram.
            elements (list[Element]): The elements in the diagram.
            requirements (list[Requirement]): The requirements in the diagram.
            links (list[Link]): The links between elements and requirements in the diagram.
            config (Optional[Config]): The configuration for the diagram. Defaults to None.
        """
        super().__init__(title, "", config)
        self.elements: list[Element] = elements if elements is not None else []
        self.requirements: list[Requirement] = (
            requirements if requirements is not None else []
        )
        self.links: list[Link] = links if links is not None else []
        self._build_script()

    def _build_script(self) -> None:
        super()._build_script()
        script: str = "\nrequirementDiagram\n"
        for element in self.elements:
            script += str(element) + "\n"
        for requirement in self.requirements:
            script += str(requirement) + "\n"

        for link in self.links:
            script += f"{link}\n"
        self.script += script


__all__ = [
    "RequirementDiagram",
    "Element",
    "Link",
    "Requirement",
    "Risk",
    "VerifyMethod",
    "Type",
]
